@@ -119,29 +119,36 @@ void Application::installSigHandlers() {
 }
 
 std::istream& Application::getStream() {
-	if (generic.input.empty() || generic.input[0] == "stdin") {
-		generic.input.resize(1, "stdin");
+	ProgramOptions::StringSeq& input = cmdOpts_.basic.input;
+	if (input.empty() || input[0] == "stdin") {
+		input.resize(1, "stdin");
 		return std::cin;
 	}
 	else {
 		static std::ifstream file;
 		if (file.is_open()) return file;
-		file.open(generic.input[0].c_str());
-		if (!file) { throw std::runtime_error("Can not read from '"+generic.input[0]+"'");  }
+		file.open(input[0].c_str());
+		if (!file) { throw std::runtime_error("Can not read from '"+input[0]+"'");  }
 		return file;
 	}
 }
 
-void Application::printHelp() const {
+void Application::printHelp(const ProgramOptions::OptionContext& root) {
 	printf("clasp %s\n", CLASP_VERSION);
 	printf("\nusage: %s\n", CLASP_USAGE);
-	printf("%s\n", getHelp().c_str());
-	printf("\nusage: %s\n", CLASP_USAGE);
-	printf("Default commandline:\nclasp %s\n", getDefaults().c_str());
+	ProgramOptions::FileOut out(stdout);
+	root.description(out, help > 1);
+	printf("\n\nusage: %s\n", CLASP_USAGE);
+	printf("Default commandline:\nclasp %s\n", root.defaults(strlen("clasp ")).c_str());
+	printf("[asp] %s\n", cmdOpts_.getInputDefaults(Input::SMODELS));
+	printf("[cnf] %s\n", cmdOpts_.getInputDefaults(Input::DIMACS));
+	printf("[opb] %s\n", cmdOpts_.getInputDefaults(Input::OPB));
+	printf("\nclasp is part of Potassco: %s\n", "http://potassco.sourceforge.net/#clasp");
+	printf("Get help/report bugs via : http://sourceforge.net/projects/potassco/support\n");
 	fflush(stdout);
 }
 
-void Application::printVersion() const {
+void Application::printVersion(const ProgramOptions::OptionContext&) {
 	printf("clasp %s\n", CLASP_VERSION);	
 	printf("Configured with: WITH_CLASPRE=%d WITH_THREADS=%d", WITH_CLASPRE, WITH_THREADS);
 #ifndef DISABLE_MULTI_THREADING
@@ -172,7 +179,7 @@ void Application::printTemplate() const {
 	}
 }
 void Application::printWarnings() const {
-	for (StringSeq::const_iterator it = messages.warning.begin(); it != messages.warning.end(); ++it) {
+	for (ProgramOptions::StringSeq::const_iterator it = messages.warning.begin(); it != messages.warning.end(); ++it) {
 		WARNING_OUT(it->c_str());
 	}
 }
@@ -185,7 +192,7 @@ int Application::run(int argc, char** argv) {
 		ERROR_OUT("*** clasp", messages.error.c_str());
 		return S_ERROR;
 	}
-	if ((generic.help&&(printHelp(),1)) || (generic.version&&(printVersion(),1))) {
+	if (help || version) {
 		return EXIT_SUCCESS;
 	}
 	if (claspre_.listFeatures) {
@@ -201,11 +208,15 @@ int Application::run(int argc, char** argv) {
 	try {
 		StreamInput input(getStream(), detectFormat(getStream()));
 		cmdOpts_.applyDefaults(input.format());
+		if (cmdOpts_.basic.maxSat) {
+			input.format() == Input::DIMACS
+				? input.maxSat(true)
+				: warning("'--opt-sat' ignored for current input!");			
+		}
 		configureOutput(input.format());
 		if (cmdOpts_.basic.stats > 1) {
 			config_.master()->solver().stats.enableJumpStats();
 		}
-		input.maxSat(config_.opt.sat);
 		ClaspFacade clasp; facade_ = &clasp;
 		cpuTotalTime_.start();
 		clasp.solve(input, config_, this);
@@ -233,38 +244,14 @@ int Application::exception(int status, const char* what) {
 	ERROR_OUT("*** clasp", what);
 	if (facade_) {
 		cpuTotalTime_.stop();
-		printResult(stopped_);
+		if (facade_->state() == ClaspFacade::state_solve) {
+			printResult(stopped_);
+		}
 	}
 	fflush(stdout);
 	fflush(stderr);
 	_exit(status);
 }
-
-bool Application::validateDeprecated(const ProgramOptions::ParsedOptions& vm) {
-	struct P { const char* n; bool* v; int m; };
-	P deps[3] = {
-		{"solution-recording", &solu_, GlobalOptions::enum_record}, 
-		{"brave", &brave_, GlobalOptions::enum_brave}, 
-		{"cautious", &cautious_, GlobalOptions::enum_cautious}
-	}; 
-	for (uint32 i = 0; i != sizeof(deps)/sizeof(P); ++i) {
-		if (vm.count(deps[i].n)) {
-			messages.warning.push_back(std::string("Option '")+deps[i].n+"' is deprecated! Use '--enum-mode' instead.");
-			if (*deps[i].v) {
-				if (config_.enumerate.mode == GlobalOptions::enum_auto) {
-					config_.enumerate.mode = static_cast<GlobalOptions::EnumMode>(deps[i].m);
-				}
-				else if (config_.enumerate.mode != deps[i].m) {
-					messages.error = "Option '";
-					messages.error += deps[i].n;
-					messages.error += "' can't be used with other 'enum-mode'";
-					return false;
-				}
-			}
-		}
-	}
-	return true;
-}
 /////////////////////////////////////////////////////////////////////////////////////////
 // State & Result functions
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -372,7 +359,7 @@ void Application::event(const Solver& s, ClaspFacade::Event e, ClaspFacade& f) {
 // Creates output object suitable for given input format
 void Application::configureOutput(Input::Format f) {
 	if (config_.enumerate.onlyPre) {
-		generic.verbose = 0;
+		cmdOpts_.basic.verbose = 0;
 	}
 	if (config_.consequences()) {
 		if (cmdOpts_.basic.quiet.first == 3) {
@@ -391,15 +378,15 @@ void Application::configureOutput(Input::Format f) {
 			else if (f == Input::SMODELS && cmdOpts_.basic.outf == BasicOptions::out_comp) {
 				outFormat = DefaultOutput::format_aspcomp;
 			}
-			out_.reset(new DefaultOutput(generic.verbose, cmdOpts_.basic.quiet, outFormat, cmdOpts_.basic.ifs));
+			out_.reset(new DefaultOutput(cmdOpts_.basic.verbose, cmdOpts_.basic.quiet, outFormat, cmdOpts_.basic.ifs));
 		}
 		else {
-			out_.reset(new JsonOutput(generic.verbose, cmdOpts_.basic.quiet));
+			out_.reset(new JsonOutput(cmdOpts_.basic.verbose, cmdOpts_.basic.quiet));
 		}
 	}
 	else if (f == Input::SMODELS) {
 		// claspre output
-		out_.reset(claspre_.createOutput(generic.verbose, cmdOpts_.basic.quiet, DefaultOutput::format_asp));
+		out_.reset(claspre_.createOutput(cmdOpts_.basic.verbose, cmdOpts_.basic.quiet, DefaultOutput::format_asp));
 		config_.setThreads(1);
 		if (!claspre_.hasLimit) {
 			config_.master()->solve.limits = SolveLimits(500,20);
@@ -411,7 +398,7 @@ void Application::configureOutput(Input::Format f) {
 	if (out_->verbosity() > 1) {
 		config_.enumerate.progress = out_.get();
 	}
-	out_->init(std::string("clasp version ")+CLASP_VERSION, generic.input[0]);
+	out_->init(std::string("clasp version ")+CLASP_VERSION, cmdOpts_.basic.input[0]);
 }
 void Application::model(const Solver& s, const Enumerator& e, bool cons) {
 	if (!cons) { out_->reportModel(s, e); }
