@@ -51,14 +51,12 @@
 /////////////////////////////////////////////////////////////////////////////////////////
 namespace Clasp {
 	
-class Application : public AppOptions, public ClaspFacade::Callback {
+class Application : public ProgramOptions::AppOptions, public ClaspFacade::Callback {
 public:
 	static Application& instance();    // returns the singleton instance
 	static void sigHandler(int sig);   // signal/timeout handler
 	void   installSigHandlers();       // adds handlers for SIGINT, SIGTERM, SIGALRM
 	int    run(int argc, char** argv); // "entry-point"
-	void   printHelp()    const;
-	void   printVersion() const;
 	void   printTemplate()const;
 	void   printWarnings()const;
 private:
@@ -67,31 +65,21 @@ private:
 	const Application& operator=(const Application&);
 	// -------------------------------------------------------------------------------------------
 	// AppOptions interface
-	void initOptions(ProgramOptions::OptionContext& root, ProgramOptions::OptionGroup& hidden) {
+	void printHelp(const ProgramOptions::OptionContext& root)    ;
+	void printVersion(const ProgramOptions::OptionContext& root) ;
+	void initOptions(ProgramOptions::OptionContext& root) {
 		claspre_.initOptions(root);
 		cmdOpts_.setConfig(&config_);
-		cmdOpts_.initOptions(root, hidden);
-		// DEPRECATED options - remove in next version
-		ProgramOptions::OptionGroup deprecated("Clasp - DEPRECATED options");
-		deprecated.addOptions()		
-			("brave"              , ProgramOptions::flag(brave_)    , "Use --enum-mode=brave")
-			("cautious"           , ProgramOptions::flag(cautious_) , "Use --enum-mode=cautious")
-			("solution-recording" , ProgramOptions::flag(solu_)     , "Use --enum-mode=record\n")
-		;
-		root.add(deprecated);
+		cmdOpts_.initOptions(root);
 		ProgramOptions::OptionGroup basic("Basic Options");
 		basic.addOptions()
-			("lemma-out" , ProgramOptions::storeTo(lemmaOut_), "Write learnt lemmas to %A on exit", "<file>")
-			("lemma-in"  , ProgramOptions::storeTo(lemmaIn_) , "Read additional clauses from %A"  , "<file>")
+			("lemma-out" , ProgramOptions::storeTo(lemmaOut_)->arg("<file>"), "Write learnt lemmas to %A on exit")
+			("lemma-in"  , ProgramOptions::storeTo(lemmaIn_)->arg("<file>"), "Read additional clauses from %A")
 		;
 		root.add(basic);
-		generic.verbose = 1;
-		const ProgramOptions::Option& o = **root.find("verbose");
-		o.value()->defaultValue("1");
 	}
-	bool validateOptions(const ProgramOptions::ParsedOptions& vm, Messages& m) {
-		return claspre_.validateOptions(vm) && cmdOpts_.validateOptions(vm, m)
-			&& validateDeprecated(vm);
+	bool validateOptions(const ProgramOptions::ParsedOptions& vm, ProgramOptions::Messages& m) {
+		return claspre_.validateOptions(vm) && cmdOpts_.validateOptions(vm, m);
 	}
 	// -------------------------------------------------------------------------------------------
 	// ClaspFacade::Callback interface
@@ -106,7 +94,6 @@ private:
 	void readLemmas();
 	void writeLemmas();
 	int  exception(int status, const char* what);
-	bool validateDeprecated(const ProgramOptions::ParsedOptions& vm);
 	// -------------------------------------------------------------------------------------------  
 	// Status information & output
 	void configureOutput(Input::Format f);
@@ -125,10 +112,6 @@ private:
 	ClaspFacade*                  facade_;
 	std::auto_ptr<OutputFormat>   out_;
 	volatile sig_atomic_t         stopped_;
-	// DEPRECATED options - remove in next version
-	bool                          brave_;
-	bool                          cautious_;
-	bool                          solu_;
 };
 
 }
