@@ -39,17 +39,26 @@ namespace Clasp {
 /////////////////////////////////////////////////////////////////////////////////////////
 // Function for mapping positional options
 bool parsePositional(const std::string& s, std::string& out);
+struct StringToEnum {
+	const char* str; // string value
+	int         ev;  // corresponding enum value
+};
+typedef StringToEnum EnumMap;
 
 // Group "Basic Options"
 struct BasicOptions {
 	BasicOptions();
 	typedef std::pair<int, int> QPair;
+	typedef ProgramOptions::StringSeq StringSeq;
 	static bool mapIfs(const std::string& s, char& ifs);
-	void initOptions(ProgramOptions::OptionContext& root, ProgramOptions::OptionGroup& hidden);
+	void initOptions(ProgramOptions::OptionContext& root);
+	StringSeq   input;  // list of input files - only first used!
+	int         verbose;// verbosity level
 	int         timeout;// timeout in seconds (default: none=-1)
 	QPair       quiet;  // configure printing of models and optimization values
 	int         stats;  // print statistics
 	int         outf;   // output format
+	bool        maxSat; // interpret dimacs as MaxSAT problem
 	char        ifs;    // field separator
 	enum OutputFormat {
 		out_def     = 0,
@@ -64,14 +73,15 @@ struct BasicOptions {
 struct GeneralOptions {
 	GeneralOptions() : config(0), satPreDefault(true) {}
 	static bool parseEnumMode(const std::string& s, GlobalOptions::EnumMode& m); 
-	static bool mapVec(GeneralOptions*, const std::string&, const std::vector<int>* v);
-	static bool mapCustom(GeneralOptions*, const std::string& k, const std::string* v);
-	static bool mapThreads(GeneralOptions*, const std::string&, const int* v);
-	static bool parseExtRule(const std::string& s, ProgramBuilder::ExtendedRuleMode& i);
-	void initOptions(ProgramOptions::OptionContext& root, ProgramOptions::OptionGroup& hidden);
-	bool validateOptions(const ProgramOptions::ParsedOptions& parsed, Messages&);
-	ClaspConfig*   config;
-	bool           satPreDefault;
+	static bool mapMyVec(GeneralOptions*, const std::string&, const std::vector<int>* v);
+	static bool mapThreadOpts(GeneralOptions*, const std::string& k, const std::string& v);
+	void initOptions(ProgramOptions::OptionContext& root);
+	bool validateOptions(const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
+	std::vector<int>  myVec;
+	ClaspConfig*      config;
+	bool              satPreDefault;
+	static const EnumMap enumModes[];
+	static const EnumMap extRules[];
 };
 
 // Groups "Clasp - Search Options" and "Clasp - Lookback Options"
@@ -79,26 +89,24 @@ struct GeneralOptions {
 // and ClaspConfig::solver
 struct SearchOptions {
 	SearchOptions() : local(0), solverOpts(0), solveOpts(0) {}
-	void initOptions(ProgramOptions::OptionContext& root, ProgramOptions::OptionGroup& hidden);
-	bool validateOptions(const ProgramOptions::ParsedOptions& parsed, Messages&);
+	void initOptions(ProgramOptions::OptionContext& root);
+	bool validateOptions(const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
 	// value parsing 
 	static bool parseHeuristic(const std::string& s, std::string&);
-	static bool parseLookahead(const std::string& s, HeuristicOptions::LookaheadType& i);
-	static bool parseAntes(const std::string& s, SolverStrategies::CflMinAntes& i);
-	static bool parseLoops(const std::string& s, LocalOptions::LoopMode& i);
 	static bool parseSchedule(const std::string& s, ScheduleStrategy& sched);
 	// value mapping
-	static bool mapCustom(SearchOptions*, const std::string&, const std::string* v);
-	static bool mapFlag(SearchOptions* this_, const std::string& flag, const bool* val);
-	static bool mapInt(SearchOptions* this_, const std::string& key, const int* val);
-	static bool mapRandFreq(SearchOptions* this_, const std::string& s, const double* val);
-	static bool mapRandProb(SearchOptions* this_, const std::string& key, const std::string* value);
-	static bool mapLimits(SearchOptions* this_, const std::string& key, const std::string* value);
+	static bool mapCustom(SearchOptions*  , const std::string& opt, const std::string& v);
+	static bool mapNumber(SearchOptions*  , const std::string& opt, const double* v);
+	static bool mapRandOpts(SearchOptions*, const std::string& opt, const std::string& v);
 	LocalOptions*      local;
 	SolverStrategies*  solverOpts;
 	SolveParams*       solveOpts;
+	double             aNumber;
+	static const EnumMap lookTypes[];
+	static const EnumMap anteTypes[];
+	static const EnumMap loopTypes[];
 };
-
+	
 #define CLASP_DEFAULT_PORTFOLIO_SIZE 9
 extern const char* portfolio_g;
 
@@ -112,13 +120,14 @@ public:
 	// Must be called once before option parsing begins
 	void setConfig(ClaspConfig* config);
 
-	void initOptions(ProgramOptions::OptionContext& root, ProgramOptions::OptionGroup& hidden);
-	bool validateOptions(const ProgramOptions::ParsedOptions& parsed, Messages&);
+	void initOptions(ProgramOptions::OptionContext& root);
+	bool validateOptions(const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
 	void applyDefaults(Input::Format f);
+	const char* getInputDefaults(Input::Format f) const;
 private:
 	bool parsePortfolio(std::istream& in, uint32 max, std::string& mem) const;
-	bool populateThreadConfigs(const char* port, const ProgramOptions::ParsedOptions& parsed, Messages&);
-	bool parseConfig(const char* p, uint32 id, const ProgramOptions::ParsedOptions& parsed, Messages&);
+	bool populateThreadConfigs(const char* port, const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
+	bool parseConfig(const char* p, uint32 id, const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
 	GeneralOptions mode;
 	SearchOptions  search;
 	ClaspConfig*   config;
