@@ -892,10 +892,9 @@ void Options::initOptions(ProgramOptions::OptionContext& root) {
 	using namespace ProgramOptions;
 	OptionGroup pre("Claspre Options");
 	pre.addOptions()
-		("list-features"   , flag(listFeatures),"Print feature names (in --features=C1 format/order)")
+		("list-features"   , flag(listFeatures),"Print feature names (in --features=C1 format)")
 		("features"   , storeTo(features, &Options::mapFormat),
 			"Print features in selected format\n"
-			"      Default: no\n"
 			"      Valid: V, C1, C2, no\n"
 			"        V : Print features in human-readable format\n"
 			"        C1: Print in compact format; iterated features in separate lines\n"
