@@ -26,7 +26,7 @@
 #endif
 
 #if !defined(CLASP_VERSION)
-#define CLASP_VERSION "2.0.3"
+#define CLASP_VERSION "2.0.4"
 #endif
 #if !defined(CLASP_LEGAL)
 #define CLASP_LEGAL \
@@ -146,12 +146,11 @@ public:
 	ThreadOptions thread; /**< Options for parallel solving */
 	EqOptions     eq;     /**< options for equivalence preprocessing */
 	struct Optimize {     /**< Optimization options */
-		Optimize() : hierarch(0), no(false), all(false), sat(false) {}
+		Optimize() : hierarch(0), no(false), all(false) {}
 		WeightVec vals;     /**< initial values for optimize statements */
 		int    hierarch;    /**< use hierarchical optimization scheme */
 		bool   no;          /**< ignore optimize statements */
 		bool   all;         /**< compute all models <= vals */
-		bool   sat;         /**< force max sat */
 	}    opt;
 	struct EnumOptions {  /**< Enumeration options */
 		EnumOptions() : progress(0), numModels(-1), mode(enum_auto), projectOpts(7), project(false)
