@@ -315,22 +315,6 @@ public:
 	 */
 	virtual bool isModel(Solver& s);
 
-	//! checks whether there is a model that is symmetric to the current model
-	/*!
-	 * The function shall check for and (optionally) expand symmetric models, 
-	 * i.e. models that differ only in the assignment of variables outside of the 
-	 * solver's assignment. 
-	 * \param s      The solver in which this object is used
-	 * \param expand If true, symmetric models shall be expanded. Otherwise they
-	 *               they shall be ignored.
-	 * \pre the current assignment is a model
-	 * \return 
-	 *   - true, if there are symmetric models and expand is true
-	 *   - false, otherwise.
-	 * \note the default implementation always returns false
-	 */ 
-	virtual bool nextSymModel(Solver& s, bool expand);
-
 	//! propagates the current assignment until a fixpoint is reached w.r.t this post propagator
 	/*!
 	 * This function implements a basic propagation loop that calls
@@ -345,10 +329,10 @@ public:
 	 * \pre    the assignment is fully propagated.
 	 * \post   a fixpoint was reached or propagation led to a conflict.
 	 * \note   The default implementation implements the following loop:
-	 *   while (propagate() && s.queueSize() > 0) 
+	 *   REPEAT:
+	 *     if (!propagate())            return false;
+	 *     if (s.queueSize() == 0)      return true;
 	 *     if (!s.propagateUntil(this)) return false;
-	 *   return true
-	 * \note   
 	 */
 	virtual bool propagateFixpoint(Solver& s);
 protected:
