@@ -159,7 +159,7 @@ public:
 	/*!
 	 * The return value determines how search should proceed.
 	 * If enumerate_continue is returned, the enumerator has
-	 * removed at least one decision level and search should
+	 * removed at least one decision level and search may
 	 * continue from the new level.
 	 * Otherwise, the search shall be stopped.
 	 * \pre The solver contains a model and DL = s.decisionLevel()
@@ -177,13 +177,14 @@ public:
 	/*!
 	 * Adds new information from the recent model and strengthens
 	 * any conditional knowledge in s.
-	 * \pre backtrackFromModel(s) was called and returned enumerate_continue
-	 * \return enumerate_stop_complete if an unresolvable conflict was found.
-	 *         Otherwise enumerate_continue
+	 * \pre backtrackFromModel(s) was called
+	 * \return false if an unresolvable conflict was found.
+	 *
 	 * \note The function is concurrency-safe, i.e. can be called
 	 *       concurrently by different solvers.
+	 *
 	 */
-	Result continueFromModel(Solver& s);
+	bool continueFromModel(Solver& s, bool allowModelHeuristic = true);
 
 	//! updates solver s with new model-related information
 	/*!
@@ -222,7 +223,6 @@ public:
 protected:
 	uint32 getHighestActiveLevel() const { return activeLevel_; }
 	uint32 nextUpdate()                  { return (updates_ += 2); }
-	Result backtrackModel(Solver& s, bool cont);
 	virtual EnumeratorConstraint* doInit(SharedContext& s, uint32 numThreads, bool start) = 0;
 	virtual bool backtrack(Solver& s) = 0;
 	virtual bool updateConstraint(Solver& s, bool disjoint) = 0;
@@ -238,7 +238,6 @@ private:
 	std::atomic<uint32> updates_;
 	uint32              activeLevel_;
 	bool                restartOnModel_;
-	bool                update_;
 };
 
 class NullEnumerator : public Enumerator {
