@@ -205,18 +205,19 @@ struct SolveParams {
 	 * \param runs number of initial randomized-runs
 	 * \param cfl number of conflicts comprising one randomized-run
 	 */
-	void setRandomizeParams(uint32 runs, uint32 cfls) {
-		if (runs > 0 && cfls > 0) {
-			init.randRuns = (uint16)std::min(runs, (1u<<16)-1);
-			init.randConf = (uint16)std::min(cfls, (1u<<16)-1);
-		}
+	bool setRandomizeParams(uint32 runs, uint32 cfls) {
+		if (!runs || !cfls) { runs = cfls = 0; }
+		init.randRuns = (uint16)std::min(runs, (1u<<16)-1);
+		init.randConf = (uint16)std::min(cfls, (1u<<16)-1);
+		return true;
 	}
 
 	//! sets the probability with which choices are made randomly instead of with the installed heuristic.
-	void setRandomProbability(double p) {
+	bool setRandomProbability(double p) {
 		if (p >= 0.0 && p <= 1.0) {
 			randFreq_ = p;
 		}
+		return randFreq_ == p;
 	}
 	// accessors
 	uint32  randRuns()          const { return init.randRuns; }
