@@ -835,7 +835,7 @@ public:
 	}
 	
 	//! removes c from the watch-list of the decision level dl
-	void removeUndoWatch(uint32 dl, Constraint* c);
+	bool removeUndoWatch(uint32 dl, Constraint* c);
 
 	void initSavedValue(Var v, ValueRep val) {
 		assert(validVar(v));
