@@ -50,11 +50,13 @@ class SharedLiterals;
  */
 struct ParallelStats {
 	ParallelStats() { reset(); }
-	double  cpuTime; /**< (Estimated) cpu time of the current solver */
-	uint64  splits;  /**< Number of split requests handled */
-	uint64  gps;     /**< Number of guiding paths received */
-	uint64  gpLits;  /**< Sum of literals in received guiding paths */
-	uint64  shared;  /**< Number of nogoods shared */
+	double  cpuTime;  /**< (Estimated) cpu time of the current solver */
+	uint64  splits;   /**< Number of split requests handled */
+	uint64  gps;      /**< Number of guiding paths received */
+	uint64  gpLits;   /**< Sum of literals in received guiding paths */
+	uint64  shared;   /**< Number of nogoods shared */
+	uint64  imps;     /**< Number of initial implications from shared */
+	uint64  jumps;    /**< Sum of backjumps needed to integrate new implications */
 	bool    terminated;
 	void    reset() { 
 		std::memset(this, 0, sizeof(*this)); 
@@ -62,14 +64,15 @@ struct ParallelStats {
 		terminated = false;
 	}
 	void    accu(const ParallelStats& o) {
-		cpuTime += o.cpuTime;
-		splits  += o.splits;
-		gps     += o.gps;
-		gpLits  += o.gpLits;
-		shared  += o.shared;
+		cpuTime  += o.cpuTime;
+		splits   += o.splits;
+		gps      += o.gps;
+		gpLits   += o.gpLits;
+		shared   += o.shared;
+		imps     += o.imps;
+		jumps    += o.jumps;
 		terminated |= o.terminated;
 	}
-
 	void newGP(LitVec::size_type length) {
 		++gps;
 		gpLits += length;
@@ -416,17 +419,20 @@ inline void releaseVec(WatchList& w) { w.clear(true); }
 struct ReasonStore32 : PodVector<Antecedent>::type {
 	uint32  dataSize() const     { return (uint32)size(); }
 	void    dataResize(uint32)   {}
-	uint32  data(uint32 v) const { return static_cast<uint32>((*this)[v].asUint()>>32);}
+	uint32  data(uint32 v) const { return decode((*this)[v]);}
 	void    setData(uint32 v, uint32 data) { encode((*this)[v], data); }
-	static  void encode(Antecedent& a, uint32 data) {
+	static  void   encode(Antecedent& a, uint32 data) {
 		a.asUint() = (uint64(data)<<32) | static_cast<uint32>(a.asUint());
 	}
+	static  uint32 decode(const Antecedent& a) {
+		return static_cast<uint32>(a.asUint()>>32);
+	}
 	struct value_type {
 		value_type(const Antecedent& a, uint32 d) : ante_(a) {
-			if (d != UINT32_MAX) { encode(ante_, d); }
+			if (d != UINT32_MAX) { encode(ante_, d); assert(data() == d && ante_.type() == Antecedent::generic_constraint); }
 		}
 		const Antecedent& ante() const { return ante_;      }
-		      uint32      data() const { return UINT32_MAX; }
+		      uint32      data() const { return ante_.type() == Antecedent::generic_constraint ? decode(ante_) : UINT32_MAX; }
 		Antecedent ante_;
 	};
 };
