@@ -56,8 +56,9 @@ DecisionHeuristic* HeuristicOptions::createHeuristic() const {
 	else if (name == "none") {
 		heu = new SelectFirst();
 	}
-	if (lookahead != Lookahead::no_lookahead || lookaheadNum != -1) {
-		return new UnitHeuristic(lookahead, nant, heu, lookaheadNum);
+	else if (name == "unit") {
+		assert(lookahead != Lookahead::no_lookahead);
+		heu = new UnitHeuristic(lookahead, nant);
 	}
 	return heu;
 }
@@ -80,8 +81,17 @@ HeuristicOptions& LocalOptions::heuristic() {
 void LocalOptions::applyHeuristic() {
 	if (heuristic_) {
 		DecisionHeuristic* h = heuristic_->createHeuristic();
+		if (heuristic_->name != "unit" && heuristic_->lookahead != Lookahead::no_lookahead) {
+			if (heuristic_->lookaheadNum != -1) {
+				solve.init.initLook = (uint16)std::min(heuristic_->lookaheadNum+1, (1<<16)-1);
+				solve.init.lookType = (uint16)heuristic_->lookahead;
+			}
+			else {
+				solver_->addPost(new Lookahead(heuristic_->lookahead));
+			}
+		}
 		delete heuristic_;
-		heuristic_ = 0;		
+		heuristic_ = 0;
 		solver_->strategies().heuristic.reset(h);
 	}
 }
@@ -119,6 +129,7 @@ void LocalOptions::initFrom(const LocalOptions& other) {
 	st->strengthenRecursive = other.solver_->strategies().strengthenRecursive;
 	st->randomWatches= other.solver_->strategies().randomWatches;
 	st->setCompressionStrategy(other.solver_->strategies().compress());
+	st->heuOpts      = other.solver_->strategies().heuOpts;
 	
 	solve   = other.solve;
 	loopRep = other.loopRep;
@@ -137,9 +148,9 @@ Enumerator* GlobalOptions::initEnumerator(Enumerator::Report* r) {
 	ModelEnumerator* e = 0;
 	Enumerator* ret    = 0;
 	if (consequences()) {
-		ret = new CBConsequences(enumerate.brave ? CBConsequences::brave_consequences : CBConsequences::cautious_consequences);
+		ret = new CBConsequences(enumerate.mode == enum_brave ? CBConsequences::brave_consequences : CBConsequences::cautious_consequences);
 	}
-	else if (enumerate.record) {
+	else if (enumerate.mode == enum_record) {
 		ret = (e = new RecordEnumerator());
 	}
 	else {
@@ -170,11 +181,10 @@ bool ClaspConfig::validate(std::string& err) {
 			return false;
 		}
 	}
-	if (enumerate.brave && enumerate.cautious) {
-		err = "Options 'brave' and 'cautious' are mutually exclusive!";
+	if (enumerate.mode == enum_bt && enumerate.restartOnModel) { 
+		err = "Options 'restart-on-model' and 'enum-mode=bt' are mutually exclusive!";
 		return false;
 	}
-	if (enumerate.restartOnModel) { enumerate.record  = true; }
 	return true;
 }
 
@@ -231,7 +241,7 @@ ClaspFacade::ClaspFacade()
 	, graph_(0)
 	, api_(0)
 	, result_(result_unknown)
-	, state_(state_not_started)
+	, state_(num_states)
 	, step_(0)
 	, more_(true) {
 }
@@ -245,7 +255,7 @@ void ClaspFacade::init(Input& problem, ClaspConfig& config, IncrementalControl*
 	graph_  = 0;
 	api_    = 0;
 	result_ = result_unknown;
-	state_  = state_not_started;
+	state_  = num_states;
 	step_   = 0;
 	more_   = true;
 	validateWeak();
@@ -293,6 +303,13 @@ void ClaspFacade::validateWeak() {
 		warning("Supported models requires --eq=0. Disabling eq-preprocessor!");
 		config_->eq.noEq();
 	}
+	if (config_->enumerate.project && config_->enumerate.mode == GlobalOptions::enum_bt && config_->numThreads() > 1) {
+		warning("Multi-Threading disabled - Selected reasoning mode not supported!");
+		config_->setThreads(1);
+	}
+	if (config_->numThreads() > ClaspFacade::hardware_concurrency()) {
+		warning("Oversubscription: Number of threads exceeds hardware concurrency!");
+	}
 	if (config_->opt.all || config_->opt.no) {
 		config_->opt.hierarch = 0;
 	}
@@ -302,6 +319,7 @@ void ClaspFacade::validateWeak() {
 // Non-incremental solving...
 void ClaspFacade::solve(Input& problem, ClaspConfig& config, Callback* c) {
 	init(problem, config, 0, c);
+	AutoState outer(this, state_start);
 	LocalOptions* master = config.master();
 	if (!read() || !preprocess()) {
 		result_ = result_unsat;
@@ -319,6 +337,7 @@ void ClaspFacade::solve(Input& problem, ClaspConfig& config, Callback* c) {
 // Incremental solving...
 void ClaspFacade::solveIncremental(Input& problem, ClaspConfig& config, IncrementalControl& inc, Callback* c) {
 	init(problem, config, &inc, c);
+	AutoState outer(this, state_start);
 	LitVec assume;
 	LocalOptions* master = config_->master();
 	do {
@@ -401,6 +420,7 @@ bool ClaspFacade::preprocess() {
 				}
 			}
 			m = builder.build(ctx, config_->ctx.tagLiteral());
+			if (!m) { return false; }
 		}
 		if (!builder.hasRules() || (builder.numRules() == 1 && config_->opt.hierarch < 2)) {
 			config_->ctx.removeTagLiteral();
@@ -410,29 +430,7 @@ bool ClaspFacade::preprocess() {
 	if (!inc_ && api_.is_owner()) {
 		api_ = 0;
 	}
-	if (!initEnumeration(m) || !config_->ctx.endInit(config_->numThreads())) {
-		return false;
-	}
-	return true;
-}
-
-// Configures the given minimize constraint and adds it to the enumerator.
-// Optimize values that are given in config are added to min.
-bool ClaspFacade::configureMinimize(SharedMinimizeData* min) const {
-	min->setMode(config_->opt.all ? MinimizeMode_t::enumerate : MinimizeMode_t::optimize, config_->opt.hierarch);
-	if (config_->consequences()) {
-		warning("Minimize statements: Consequences may depend on enumeration order!");
-	}
-	if (config_->enumerate.project) {
-		for (const WeightLiteral* it = min->lits; !isSentinel(it->first); ++it) {
-			if ( !config_->ctx.project(it->first.var()) ) {
-				warning("Projection: Optimization values may depend on enumeration order!");
-				break;
-			}
-		}
-	}
-	config_->ctx.enumerator()->setMinimize(min, config_->opt.heu);
-	return true;
+	return config_->enumerate.onlyPre || (initEnumeration(m) && config_->ctx.endInit(config_->numThreads()));
 }
 
 // Finalizes initialization of model enumeration.
@@ -440,37 +438,48 @@ bool ClaspFacade::configureMinimize(SharedMinimizeData* min) const {
 // sts the number of models to compute and adds warnings
 // if this number conflicts with the preferred number of the enumerator.
 bool ClaspFacade::initEnumeration(SharedMinimizeData* min)  {
-	Enumerator* e = step_ == 0 ? config_->initEnumerator(this) : config_->ctx.enumerator();
-	config_->ctx.addEnumerator(e);
 	GlobalOptions::EnumOptions& opts = config_->enumerate;
+	Enumerator* e                    = config_->ctx.enumerator();
+	MinimizeMode minMode             = !min || config_->opt.all ? MinimizeMode_t::enumerate : MinimizeMode_t::optimize;
 	if (step_ == 0) {
-		if (min && !configureMinimize(min)) {
-			return false;
+		GlobalOptions::EnumMode autoMode = GlobalOptions::enum_bt;
+		if (opts.restartOnModel) {
+			autoMode = GlobalOptions::enum_record;
 		}
-		uint32 defM = !e->minimize() && !config_->consequences();
-		if (opts.numModels == -1) { 
-			opts.numModels = defM; 
+		if (minMode == MinimizeMode_t::optimize && !opts.project) {
+			autoMode = GlobalOptions::enum_record;
 		}
-		else if (opts.numModels > 0 && defM == 0) {
-			if (config_->consequences()) {
-				warning("Option '--number' not 0: last model may not cover consequences!");
-			}
-			if (e->minimize()) {
-				warning("Option '--number' not 0: Optimality of last model not guaranteed!");
+		if (opts.project && config_->numThreads() > 1) {
+			autoMode = GlobalOptions::enum_record;
+		}
+		uint32 autoModels = !min && !config_->consequences();
+		if (autoModels == 0 && opts.numModels > 0) {
+			if (config_->consequences())            { warning("Option '--number' not 0: last model may not cover consequences!");   }
+			if (minMode == MinimizeMode_t::optimize){ warning("Option '--number' not 0: Optimality of last model not guaranteed!"); }
+		}
+		if (opts.numModels == -1)      { opts.numModels = autoModels; }	
+		if (config_->consequences() && minMode == MinimizeMode_t::optimize) {
+			warning("Minimize statements: Consequences may depend on enumeration order!");
+		}
+		if (opts.project && minMode == MinimizeMode_t::optimize) {
+			for (const WeightLiteral* it = min->lits; !isSentinel(it->first); ++it) {
+				if ( !config_->ctx.project(it->first.var()) ) {
+					warning("Projection: Optimization values may depend on enumeration order!");
+					break;
+				}
 			}
 		}
-		if (config_->numThreads() > 1) {
-			bool forceST = opts.project && !opts.record;
-			if (forceST) {
-				warning("Multi-Threading disabled - Selected reasoning mode not supported!");
-				config_->setThreads(1);
-			}
-			else if (config_->numThreads() > ClaspFacade::hardware_concurrency()) {
-				warning("Oversubscription: Number of threads exceeds hardware concurrency!");
-			}
+		if (config_->enumerate.mode == GlobalOptions::enum_auto) {
+			config_->enumerate.mode = autoMode;
 		}
+		e = config_->initEnumerator(this);
 	}
-	config_->ctx.enumerator()->enumerate(opts.numModels);
+	if (min) {
+		min->setMode(minMode, config_->opt.hierarch);
+		e->setMinimize(min);
+	}
+	e->enumerate(opts.numModels);
+	config_->ctx.addEnumerator(e);
 	return true;
 }
 
