@@ -730,41 +730,33 @@ ClauseHead::BoolPair Clause::strengthen(Solver& s, Literal p, bool toShort) {
 	Literal* it = std::find(head_, eoh, p);
 	BoolPair ret(false, false);
 	if (it != eoh) {
-		if (it == head_ || it == head_+1) {
-			// update watch
-			uint32 w = it != head_;
-			head_[w] = head_[2];
+		if (it != head_+2) { // update watch
+			*it = head_[2];
 			s.removeWatch(~p, this);
-			Literal* best = head_+w;
-			for (it = t.first; it != eot && s.isFalse(*best); ++it) {
-				if (!s.isFalse(*it) || s.level(it->var()) > s.level(best->var())) { 
-					best = it; 
+			Literal* best = it, *n;
+			for (n = t.first; n != eot && s.isFalse(*best); ++n) {
+				if (!s.isFalse(*n) || s.level(n->var()) > s.level(best->var())) { 
+					best = n; 
 				}
 			}
-			std::swap(head_[w], *best);
-			s.addWatch(~head_[w], ClauseWatch(this));
+			std::swap(*it, *best);
+			s.addWatch(~*it, ClauseWatch(this));
+			it = head_+2;
 		}	
 		// replace cache literal with literal from tail
-		head_[2]  = *t.first;
-		eot       = removeFromTail(s, t.first, eot);
+		if ((*it  = *t.first) != negLit(0)) {
+			eot     = removeFromTail(s, t.first, eot);
+		}
 		ret.first = true;
 	}
-	else if ((it = std::find(t.first, eot, p)) != eot || (contracted() && *it == p)) {
-		eot = removeFromTail(s, it, eot);
+	else if ((it = std::find(t.first, eot, p)) != eot) {
+		eot       = removeFromTail(s, it, eot);
 		ret.first = true;
 	}
 	else if (contracted()) {
-		assert(*it != p);
-		for (; !it++->watched();) {
-			if (*it == p) {
-				ret.first = true;
-				Literal* c = it-1;
-				while (!it->watched()) { *++c = *++it; }
-				c->watch();
-				break;
-			}
-		}
-		eot = it;
+		for (; *it != p && !it->watched(); ++it) { ; }
+		ret.first = *it == p;
+		eot       = *it == p ? removeFromTail(s, it, eot) : it + 1;
 	}
 	if (ret.first && ~p == s.sharedContext()->tagLiteral()) {
 		clearTagged();
@@ -774,35 +766,29 @@ ClauseHead::BoolPair Clause::strengthen(Solver& s, Literal p, bool toShort) {
 }
 
 Literal* Clause::removeFromTail(Solver& s, Literal* it, Literal* end) {
+	assert(it != end || contracted());
 	if (!contracted()) {
-		if (it != end) {
-			*it = *--end;
-			if (!isSmall()) { 
-				tail_.data.setSize(tail_.data.size()-1);
-				tail_.data.setIdx(3);
-			}
-		}
+		*it  = *--end;
 		*end = negLit(0);
-		return end;
-	}
-	else if (!end->watched()) {
-		Literal wOld = *end;
-		Literal* j   = it;
-		do { *j = *++it; } while (!j++->watched()); 
-		*j = negLit(0);
-		if (s.level(wOld.var()) != s.level(end->var())) {
-			s.removeUndoWatch(s.level(wOld.var()), this);
-			s.addUndoWatch(s.level(end->var()), this);
+		if (!isSmall()) { 
+			tail_.data.setSize(tail_.data.size()-1);
+			tail_.data.setIdx(3);
 		}
-		return j;
 	}
 	else {
-		assert(it == end);
-		s.removeUndoWatch(s.level(end->var()), this);
-		tail_.data.clearContracted();
-		*end = negLit(0);
-		return end;
+		uint32 uLev  = s.level(end->var());
+		Literal* j   = it;
+		while ( !j->watched() ) { *j++ = *++it; }
+		*j           = negLit(0);
+		uint32 nLev  = s.level(end->var());
+		if (uLev != nLev && s.removeUndoWatch(uLev, this) && nLev != 0) {
+			s.addUndoWatch(nLev, this);
+		}
+		if (j != end) { (j-1)->watch(); }
+		else          { tail_.data.clearContracted(); }
+		end = j;
 	}
+	return end;
 }
 uint32 Clause::size() const {
 	return !isSmall() 
