@@ -86,7 +86,6 @@ void Enumerator::enumerate(uint64 m)                    { numModels_ = m; }
 void Enumerator::startInit(SharedContext& ctx) { 
 	enumerated = 0;
 	updates_   = 0;
-	update_    = false;
 	doInit(ctx, 0, true);
 }
 
@@ -102,44 +101,22 @@ Enumerator::EnumeratorConstraint* Enumerator::constraint(const Solver& s) const
 	return static_cast<Enumerator::EnumeratorConstraint*>(s.getEnumerationConstraint());
 }
 
-Enumerator::Result Enumerator::continueFromModel(Solver& s) {
-	assert(!s.hasConflict());
+
+bool Enumerator::continueFromModel(Solver& s, bool heu) {
+	if (!optimize()) { return true; }
 	s.strengthenConditional();
-	MinimizeConstraint* min = 0;
-	return !optimize() || ((min=constraint(s)->minimize())->integrateNext(s) 
-	  &&   min->modelHeuristic(s))
-		? enumerate_continue : enumerate_stop_complete;
+	MinimizeConstraint* min = constraint(s)->minimize();
+	return !s.isTrue(s.sharedContext()->tagLiteral())
+	  || (!s.hasConflict() && min->integrateNext(s) && (!heu || min->modelHeuristic(s)));
 }
 
-Enumerator::Result Enumerator::backtrackModel(Solver& s, bool cont) {
-	EnumeratorConstraint* c = constraint(s);
-	Enumerator::Result    r = Enumerator::enumerate_stop_complete;
-	if (activeLevel_ > s.rootLevel() && backtrack(s)) {
-		if (restartOnModel_) { s.undoUntil(0); }	
-		if (optimize())      { update_ = true; }
-		r = Enumerator::enumerate_continue;
-		assert(!s.hasConflict());
-	}
-	if (update_ && !s.sharedContext()->unique()) {
-		// enumerator is not trivial w.r.t current search scheme
-		// force update of other solvers
-		// - this one is up to date
-		c->setUpdates(nextUpdate());
-		update_ = false;
-	}
-	if (r == Enumerator::enumerate_continue && cont) {
-		r = continueFromModel(s);
-	}
-	if (r == Enumerator::enumerate_stop_complete) {
-		s.undoUntil(0, true);
-	}
-	return r;
-}
 Enumerator::Result Enumerator::backtrackFromModel(Solver& s, bool callContinue) {
 	assert(s.numFreeVars() == 0 && !s.hasConflict());
-	update_                 = updateModel(s);
-	activeLevel_            = mini_ ? constraint(s)->minimize()->commitCurrent(s)+1 : s.decisionLevel();
+	bool update             = updateModel(s) || optimize();
 	bool expandSym          = !ignoreSymmetric();
+	activeLevel_            = mini_ ? constraint(s)->minimize()->commitCurrent(s)+1 : s.decisionLevel();
+	Enumerator::Result    r = Enumerator::enumerate_continue;
+	EnumeratorConstraint* c = constraint(s);
 	do {
 		++enumerated;
 		if (report_) { 
@@ -153,7 +130,21 @@ Enumerator::Result Enumerator::backtrackFromModel(Solver& s, bool callContinue)
 		// assignment of atoms outside of the solver's assignment. 
 		// Typical example: vars eliminated by the SAT-preprocessor
 	} while (expandSym && s.nextSymModel());
-	return backtrackModel(s, callContinue);
+	if (activeLevel_ <= s.rootLevel() || !backtrack(s)) {
+		r = enumerate_stop_complete;
+		s.undoUntil(0, true);
+	}
+	else if (restartOnModel_) { s.undoUntil(0); }	
+	if (update && !s.sharedContext()->unique()) {
+		// enumerator is not trivial w.r.t current search scheme
+		// force update of other solvers
+		// - this one is up to date
+		c->setUpdates(nextUpdate());
+	}
+	if (callContinue && !continueFromModel(s)) {
+		r = enumerate_stop_complete;
+	}
+	return r;
 }
 
 bool Enumerator::update(Solver& s, bool disjoint) {
