@@ -64,99 +64,145 @@ bool ScoreLook::greater(Var lhs, Var rhs) const {
 /////////////////////////////////////////////////////////////////////////////////////////
 // Lookahead propagator
 /////////////////////////////////////////////////////////////////////////////////////////
-Lookahead::Lookahead()
-	: head_(posLit(0)) // dummy node
-	, undo_(posLit(0)) // dummy node
-	, last_(&head_)    // circular list
-	, pos_(&head_)     // lookahead start pos
+Lookahead::Lookahead(Type type, bool imps)
+	: nodes_(2, LitNode(posLit(0)))
+	, last_(head_id)    // circular list
+	, pos_(head_id)     // lookahead start pos
 	, top_(uint32(-2)) {
-	last_->next= &head_;
-}
-Lookahead::~Lookahead() { clear(); }
-void Lookahead::enable(Solver& s) {
-	if (top_ == uint32(-2)) {
-		s.addPost(this);
-		top_ = UINT32_MAX;
-		score.score.resize(s.numVars()+1);
+	head()->next = head_id;
+	undo()->next = UINT32_MAX;
+	if (type != hybrid_lookahead) {
+		score.mode = ScoreLook::score_max_min;
+		score.types= (type == body_lookahead)
+			? Var_t::body_var
+			: Var_t::atom_var;
 	}
+	else {
+		score.mode = ScoreLook::score_max;
+		score.types= Var_t::atom_body_var;
+	}
+	if (imps) { head()->lit.watch(); }
 }
-void Lookahead::disable(Solver& s) {
-	if (top_ != uint32(-2)) {
+
+Lookahead::~Lookahead() {  }
+
+void Lookahead::destroy(Solver* s, bool detach) {
+	if (s && detach) {
+		s->removePost(this);
 		while (saved_.size()>1) {
-			s.removeUndoWatch(uint32(saved_.size()-1), this);
-			splice(saved_.back());
+			s->removeUndoWatch(uint32(saved_.size()-1), this);
 			saved_.pop_back();
 		}
-		s.removePost(this);
-		top_ = uint32(-2);
 	}
+	PostPropagator::destroy(s, detach);
 }
+
 uint32 Lookahead::priority() const { return priority_lookahead; }
+
 void Lookahead::clear() {
 	score.clearDeps();
 	while (!saved_.empty()) {
-		if (saved_.back() != 0) {
+		if (saved_.back() != UINT32_MAX) {
 			splice(saved_.back());
 		}
 		saved_.pop_back();
 	}
-	for (LitNode* r = head_.next; r != &head_; ) {
-		LitNode* t = r;
-		r = r->next;
-		delete t;
+	LookList(2, *head()).swap(nodes_);
+	head()->next = head_id;
+	undo()->next = UINT32_MAX;
+	last_        = head_id;
+	top_         = UINT32_MAX;
+}
+
+bool Lookahead::init(Solver& s) {
+	ScoreLook& sc = score;
+	sc.clearDeps();
+	Var start     = (uint32)sc.score.size();
+	sc.score.resize(s.numVars()+1);
+	const VarType types= sc.types;
+	const bool uniform = types != Var_t::atom_body_var;
+	uint32 add    = 0;
+	uint32 nants  = 0;
+	for (Var v = start; v <= s.numVars(); ++v) {
+		if (s.value(v) == value_free && (s.sharedContext()->type(v) & types) != 0 ) {
+			++add;
+			nants += s.sharedContext()->nant(v);
+		}
 	}
-	last_       = &head_;
-	last_->next = &head_;
-	top_        = UINT32_MAX;
+	nodes_.reserve(nodes_.size() + add);
+	for (Var v = start; v <= s.numVars(); ++v) {
+		if (s.value(v) == value_free && (s.sharedContext()->type(v) & types) != 0 ) {
+			append(s.sharedContext()->preferredLiteralByType(v), uniform || s.sharedContext()->type(v) == Var_t::atom_body_var);
+		}
+	}
+	if (score.nant && add > 0 && add == nants) {
+		score.nant = false;
+	}
+	return true;
 }
 
 void Lookahead::append(Literal p, bool testBoth) {
-	last_->next = new LitNode(p);
-	last_       = last_->next;
-	last_->next = &head_;
+	node(last_)->next = static_cast<NodeId>(nodes_.size());
+	nodes_.push_back(LitNode(p));
+	last_             = node(last_)->next;
+	node(last_)->next = head_id;
 	// remember to also test ~p by setting watched-flag
-	if (testBoth) { last_->lit.watch(); }
+	if (testBoth) { node(last_)->lit.watch(); }
 }
 
 // test p and optionally ~p if necessary
 bool Lookahead::test(Solver& s, Literal p) {
 	return (score.score[p.var()].seen(p) || s.test(p, this))
-		&& (!p.watched() || score.score[p.var()].seen(~p) || s.test(~p, this));
+		&& (!p.watched() || score.score[p.var()].seen(~p) || s.test(~p, this))
+		&& (imps_.empty() || checkImps(s, p));
+}
+
+bool Lookahead::checkImps(Solver& s, Literal p) {
+	assert(!imps_.empty());
+	bool ok = true;
+	if (score.score[p.var()].testedBoth()) {
+		for (LitVec::const_iterator it = imps_.begin(), end = imps_.end(); it != end && ok; ++it) {
+			ok  = s.force(*it, posLit(0));
+		}
+	}
+	imps_.clear();
+	return ok && (s.queueSize() == 0 || s.propagateUntil(this));
 }
 
 // failed-literal detection - stop on failed-literal
 bool Lookahead::propagate(Solver& s) {
 	assert(!s.hasConflict());
-	saved_.resize(s.decisionLevel()+1, 0);
-	LitNode* undoLast = saved_[s.decisionLevel()];
-	if (undoLast == 0) {
-		undoLast = &undo_;
+	saved_.resize(s.decisionLevel()+1, UINT32_MAX);
+	uint32 undoId = saved_[s.decisionLevel()];
+	if (undoId == UINT32_MAX) {
+		undoId = undo_id;
 		if (s.decisionLevel() != 0) {
 			s.addUndoWatch(s.decisionLevel(), this);
 		}
 	}
 	score.clearDeps(); 
 	score.addDeps = true;
-	Literal p    = pos_->lit;
-	bool   ok    = s.value(p.var()) != value_free || test(s, p);
-	for (LitNode* r = pos_; r->next != pos_ && ok; ) {
-		p = r->next->lit;
+	Literal p     = node(pos_)->lit;
+	bool   ok     = s.value(p.var()) != value_free || test(s, p);
+	for (LitNode* r = node(pos_); r->next != pos_ && ok; ) {
+		p = node(r->next)->lit;
 		if (s.value(p.var()) == value_free) {
-			if (test(s, p)) { r   = r->next; }
+			if (test(s, p)) { r   = node(r->next); }
 			else            { pos_= r->next; ok = false; }
 		}
-		else if (r->next != last_ && r->next != &head_) {
+		else if (r->next != last_ && r->next != head_id) {
 			// unlink from candidate list
-			LitNode* t = r->next;
-			r->next    = t->next;
+			NodeId t       = r->next;
+			r->next        = node(t)->next;
 			// append to undo list
-			t->next        = undoLast->next;
-			undoLast->next = t;
-			undoLast       = t;
+			LitNode* u     = node(undoId);
+			node(t)->next  = u->next;
+			u->next        = t;
+			undoId         = t;
 		}
-		else { r = r->next; } // keep iterators valid; never unlink last node and dummy head
+		else { r = node(r->next); } // keep iterators valid; never unlink last node and dummy head
 	}
-	saved_.back() = undoLast;
+	saved_.back() = undoId;
 	return ok;
 }
 
@@ -170,7 +216,7 @@ bool Lookahead::propagateFixpoint(Solver& s) {
 	for (dl = s.decisionLevel(); !propagate(s); dl = s.decisionLevel()) {
 		// some literal failed
 		// resolve and propagate conflict
-		assert(dl+1 == s.decisionLevel());
+		assert(s.decisionLevel() >= dl);
 		if (!s.resolveConflict() || !s.propagateUntil(this)) {
 			ok = false;
 			score.clearDeps();
@@ -182,20 +228,24 @@ bool Lookahead::propagateFixpoint(Solver& s) {
 		// on level 0 unless we learn a new implication
 		assert(s.queueSize() == 0);
 		top_    = s.numAssignedVars();
-	}
+		tlc_    = Literal(score.best, score.score[score.best].prefSign());
+		LitVec().swap(imps_);
+	}	
 	return ok;
 }
 
 // splice list [undo_.next, ul] back into candidate list
-void Lookahead::splice(LitNode* ul) {
-	if (ul != &undo_) { 
-		assert(undo_.next);
+void Lookahead::splice(NodeId ul) {
+	assert(ul != UINT32_MAX);
+	if (ul != undo_id) { 
+		assert(undo()->next != UINT32_MAX);
 		// unlink from undo list
-		LitNode* first = undo_.next;
-		undo_.next     = ul->next;
+		LitNode* ulNode= node(ul);
+		NodeId   first = undo()->next;
+		undo()->next   = ulNode->next;
 		// splice into look-list
-		ul->next       = head_.next;
-		head_.next     = first;
+		ulNode->next   = head()->next;
+		head()->next   = first;
 	}
 }
 
@@ -203,123 +253,60 @@ void Lookahead::undoLevel(Solver& s) {
 	if (s.decisionLevel() == saved_.size()) {
 		const LitVec& a = s.trail();
 		score.scoreLits(s, &a[0]+s.levelStart(s.decisionLevel()), &a[0]+a.size());
+		if (s.decisionLevel() == static_cast<uint32>(head()->lit.watched())) {
+			const Literal* b = &a[0]+s.levelStart(s.decisionLevel());
+			if (b->watched()) {
+				// remember current DL for b
+				uint32 dist = ((&a[0]+a.size()) - b);
+				imps_.assign(b+1, b + std::min(dist, uint32(2048)));
+			}
+			else if (score.score[b->var()].testedBoth()) {
+				// all true lits in imps_ follow from both *b and ~*b 
+				// and are therefore implied
+				LitVec::iterator j = imps_.begin();
+				for (LitVec::iterator it = imps_.begin(), end = imps_.end(); it != end; ++it) {
+					if (s.isTrue(*it)) { *j++ = *it; }
+				}
+				imps_.erase(j, imps_.end());
+			}
+		}
 	}
 	else {
-		assert(s.decisionLevel()+1 == saved_.size());
-		LitNode* n = saved_.back();
+		assert(saved_.size() >= s.decisionLevel()+1);
+		saved_.resize(s.decisionLevel()+1);
+		NodeId n = saved_.back();
 		saved_.pop_back();
 		splice(n);
-		assert(last_->next == &head_);
-	}
-}
-/////////////////////////////////////////////////////////////////////////////////////////
-// Lookahead heuristic
-/////////////////////////////////////////////////////////////////////////////////////////
-UnitHeuristic::UnitHeuristic(Lookahead::Type t, bool nant, DecisionHeuristic* h, int m)
-	: look_(0)
-	, heu_(h)
-	, maxLook_(!h ? -1 : m)
-	, looks_(0)
-	, incr_(0)
-	, type_(t) 
-	, nant_(nant && t == Lookahead::atom_lookahead) {
-}
-UnitHeuristic::~UnitHeuristic() {
-	delete heu_;
-}
-
-void UnitHeuristic::startInit(const Solver& s) {
-	// disable lookahead during setup - 
-	// we re-enable it once all variables and
-	// constraints are known	
-	if (look_) { look_->disable(const_cast<Solver&>(s)); }
-	if (heu_)  { heu_->startInit(s); }
-}
-
-void UnitHeuristic::endInit(Solver& s) { 
-	if (incr_ == 2 && look_) {
-		look_->disable(s);
-		look_->destroy();
-		look_ = 0;
-	}
-	if (look_ == 0) { 
-		look_ = new Lookahead();
-		if (type_ != Lookahead::hybrid_lookahead) {
-			look_->score.mode = ScoreLook::score_max_min;
-			look_->score.types= (type_ == Lookahead::body_lookahead)
-				? Var_t::body_var
-				: Var_t::atom_var;
-		}
-		else {
-			look_->score.mode = ScoreLook::score_max;
-			look_->score.types= Var_t::atom_body_var;
-		}
-	}
-	ScoreLook& sc = look_->score;
-	sc.clearDeps();
-	Var start     = (uint32)sc.score.size();
-	sc.score.resize(s.numVars()+1);
-	const VarType types= sc.types;
-	const bool uniform = types != Var_t::atom_body_var;
-	uint32 nant = 0;
-	uint32 added= 0;
-	for (Var v = start; v <= s.numVars(); ++v) {
-		if (s.value(v) == value_free && (s.sharedContext()->type(v) & types) != 0 ) {
-			++added;
-			nant += s.sharedContext()->nant(v);
-			look_->append(s.sharedContext()->preferredLiteralByType(v), uniform || s.sharedContext()->type(v) == Var_t::atom_body_var);
-		}
-	}
-	if (nant_ && nant != added) {
-		sc.nant = true;
-	}
-	look_->enable(s);
-	if (heu_) heu_->endInit(s); 
-	if (heu_ && maxLook_ == -1 && incr_ == 0) {
-		suicide(s);
-	}
-	else {
-		looks_ = 0 - (maxLook_ == -1);
+		assert(node(last_)->next == head_id);
 	}
 }
 
-void UnitHeuristic::resurrect(const Solver& s, Var v) {
-	if ( look_ && (s.sharedContext()->type(v) & look_->score.types) != 0 ) {
-		look_->score.score.resize(s.numVars()+1);
-		look_->append(s.sharedContext()->preferredLiteralByType(v), 
-			s.sharedContext()->type(v) == Var_t::atom_body_var || type_ != Lookahead::hybrid_lookahead);
+Literal Lookahead::heuristic(Solver& s) {
+	if (s.numFreeVars() == 0) { 
+		return negLit(0);  
 	}
-	if (heu_) heu_->resurrect(s, v);
-}
-
-void UnitHeuristic::reinit(bool b) {
-	incr_ = 1 + b;
-	if (heu_) heu_->reinit(b); 
-}
-
-Literal UnitHeuristic::heuristic(Solver& s) {
-	ScoreLook& sc = look_->score;
+	ScoreLook& sc  = score;
+	Literal choice = Literal(sc.best, sc.score[sc.best].prefSign());
 	if (sc.deps.empty()) {
-		if (s.value(top_.var()) == value_free) return top_;
 		// No candidates. This can happen if the problem 
 		// contains choice rules and lookahead is not atom-based.
 		// Add remaining free vars to lookahead so that we can
 		// make an informed decision.
 		for (Var v = 1, end = s.numVars()+1; v != end; ++v) {
 			if (s.value(v) == value_free) {
-				look_->append(s.sharedContext()->preferredLiteralByType(v), true);
+				append(s.sharedContext()->preferredLiteralByType(v), true);
 			}
 		}
 		sc.types  = Var_t::atom_body_var;
 		uint32 dl = s.decisionLevel();
-		if (!look_->propagate(s)) {
+		if (!propagate(s)) {
 			Literal choice = s.decision(s.decisionLevel());
 			s.undoUntil(dl);
 			return choice;
 		}
+		choice = Literal(sc.best, sc.score[sc.best].prefSign());
 	}
-	assert(s.value(sc.best) == value_free);
-	Literal choice = Literal(sc.best, sc.score[sc.best].prefSign());
+	assert(s.value(choice.var()) == value_free);
 	if (sc.mode == ScoreLook::score_max_min) {
 		uint32 min, max;
 		sc.score[sc.best].score(max, min);
@@ -329,18 +316,18 @@ Literal UnitHeuristic::heuristic(Solver& s) {
 		do {
 			Var v        = sc.deps[i];
 			VarScore& vs = sc.score[v];
-			if (!vs.testedBoth()) {
+			if (!vs.testedBoth() && s.value(v) == value_free) {
 				uint32 vMin, vMax;
 				vs.score(vMax, vMin);
 				if (vMin == 0 || vMin > min || (vMin == min && vMax > max)) {
 					uint32 neg = vs.score(negLit(v)) > 0 ? vs.score(negLit(v)) : max+1;
 					uint32 pos = vs.score(posLit(v)) > 0 ? vs.score(posLit(v)) : max+1;
 					if (!vs.tested(negLit(v))) {
-						ok  = ok && s.test(negLit(v), look_);
+						ok  = ok && s.test(negLit(v), this);
 						neg = vs.score(negLit(v));
 					}
 					if ((neg > min || (neg == min && pos > max)) && !vs.tested(posLit(v))) {
-						ok  = ok && s.test(posLit(v), look_);
+						ok  = ok && s.test(posLit(v), this);
 						pos = vs.score(posLit(v));
 					}
 				}
@@ -360,44 +347,88 @@ Literal UnitHeuristic::heuristic(Solver& s) {
 			// literal that caused the conflict
 			sc.clearDeps();
 			assert(s.hasConflict() && s.decisionLevel() > 0);
-			choice = ~s.decision(s.decisionLevel());
+			choice = negLit(0);
 		}
 		sc.deps.clear();	
 	}
-	assert(!isSentinel(choice));
-	if (s.decisionLevel() == 0) top_ = choice;
+	if (s.decisionLevel() == 0) { tlc_ = choice; }
 	return choice;
 }
 
-Literal UnitHeuristic::doSelect(Solver& s) {
-	Literal choice;
-	if (looks_ == -1) {
-		choice = !heu_ ? heuristic(s) : heu_->doSelect(s);
-	}
-	else if (++looks_ < maxLook_) {
-		choice = heuristic(s);
-	}
-	else {
-		choice = looks_ == maxLook_ ? heuristic(s) : heu_->doSelect(s);
-		// max lookahead based decisions
-		// disable lookahead
-		looks_ = -1;
-		look_->disable(s);
-		look_->destroy();
+/////////////////////////////////////////////////////////////////////////////////////////
+// Lookahead heuristic
+/////////////////////////////////////////////////////////////////////////////////////////
+UnitHeuristic::UnitHeuristic(Lookahead::Type t, bool nant)
+	: look_(0)
+	, type_(t) 
+	, init_(false)
+	, nant_(nant && t == Lookahead::atom_lookahead) {
+}
+
+UnitHeuristic::~UnitHeuristic() { }
+
+void UnitHeuristic::endInit(Solver& s) { 
+	assert(s.decisionLevel() == 0);
+	if (init_ && look_) {
+		look_->destroy(&s, true);
 		look_ = 0;
-		if (incr_ == 0) {
-			// we won't reenable lookahead, hence
-			// no need to stick around	
-			suicide(s);
-		}
+	}
+	if (look_ == 0) { 
+		look_ = new Lookahead(type_);
+		look_->score.nant = nant_;
+		look_->init(s);
+		s.addPost(look_);
+	}
+}
+
+void UnitHeuristic::resurrect(const Solver& s, Var v) {
+	if ( look_ && (s.sharedContext()->type(v) & look_->score.types) != 0 ) {
+		look_->score.score.resize(s.numVars()+1);
+		look_->append(s.sharedContext()->preferredLiteralByType(v), 
+			s.sharedContext()->type(v) == Var_t::atom_body_var || type_ != Lookahead::hybrid_lookahead);
+	}
+}
+/////////////////////////////////////////////////////////////////////////////////////////
+// Restricted Lookahead heuristic
+/////////////////////////////////////////////////////////////////////////////////////////
+RestrictedUnit::RestrictedUnit(uint32 k, Lookahead* look, DecisionHeuristic* def) 
+	: look_(look), default_(def), numChoices_(k) {
+	assert(look && k);
+}
+
+void RestrictedUnit::decorate(Solver& s, uint32 k, Lookahead* look) {
+	s.removePost(look);
+	RestrictedUnit* unit = new RestrictedUnit(k, look, s.strategies().heuristic.release());
+	s.strategies().heuristic.reset(unit);
+}
+
+RestrictedUnit::~RestrictedUnit() {
+	delete default_;
+	delete look_;
+}
+
+void RestrictedUnit::endInit(Solver& s) {
+	destroy(s);
+	s.strategies().heuristic->endInit(s);
+}
+
+Literal RestrictedUnit::doSelect(Solver& s) {
+	s.addPost(look_);
+	Literal choice = look_->propagateFixpoint(s) ? look_->heuristic(s) : negLit(0);
+	s.removePost(look_);
+	if (!isSentinel(choice) && --numChoices_ == 0) {
+		destroy(s);
 	}
 	return choice;
 }
 
-void UnitHeuristic::suicide(Solver& s) {
-	s.strategies().heuristic.release();
-	s.strategies().heuristic.reset(heu_);
-	heu_ = 0;
+void RestrictedUnit::destroy(Solver& s) {
+	if (s.strategies().heuristic.get() == this) {
+		s.strategies().heuristic.release();
+	}
+	s.strategies().heuristic.reset(default_);
+	default_ = 0;
+	if (look_) { look_->destroy(&s, true); look_ = 0;  } 
 	delete this;
 }
 
