@@ -26,14 +26,12 @@ namespace Clasp {
 /////////////////////////////////////////////////////////////////////////////////////////
 #define STRATEGY(x) (x)
 
-MinimizeConstraint::MinimizeConstraint(SharedData* d, unsigned heuristic)
+MinimizeConstraint::MinimizeConstraint(SharedData* d)
 	: shared_(d)
 	, pos_(d->lits)
 	, undo_(0)
 	, undoTop_(0)
 	, type_(d->weights.empty() ? SINGLE_LEVEL_ARITH : MULTI_LEVEL_ARITH)
-	, signHeu_((heuristic & 1u) != 0)
-	, modelHeu_((heuristic & 2u) != 0)
 	, ownsShared_(false) {
 	sum_      = new wsum_t[numRules()*4](); // [0-numRules()) stores sum
 	opt_      = &sum_[1 * numRules()];
@@ -52,7 +50,7 @@ bool MinimizeConstraint::attach(Solver& s) {
 	uint32 numL = 0;
 	VarVec up;
 	const SharedData* d = shared_;
-	const bool heuristic= signHeu_;
+	const bool heuristic= (s.strategies().heuOpts & SolverStrategies::heu_use_static_sign) != 0;
 	for (const WeightLiteral* it = d->lits; !isSentinel(it->first); ++it, ++numL) {
 		if (s.value(it->first.var()) == value_free) {
 			s.addWatch(it->first, this, numL);
@@ -73,8 +71,9 @@ bool MinimizeConstraint::attach(Solver& s) {
 	for (WeightVec::size_type i = 0; i != up.size(); ++i) {
 		MinimizeConstraint::propagate(s, shared_->lits[up[i]].first, up[i]);
 	}
-	restoreOptimum();
-	return propagateNewOptimum(s);
+	bool hasOpt = restoreOptimum() && s.isTrue(s.sharedContext()->tagLiteral());
+	return !s.hasConflict()
+		&& (!hasOpt || (propagateImpl(s, propagate_new_opt) && integrateNext(s)));
 }
 
 void MinimizeConstraint::destroy(Solver* s, bool detach) {
@@ -126,7 +125,7 @@ Constraint::PropResult MinimizeConstraint::propagate(Solver& s, Literal, uint32&
 // computes the set of literals implying p and returns 
 // the highest decision level of that set
 // PRE: p is implied on highest undo level
-uint32 MinimizeConstraint::computeImplicationSet(Solver& s, const WeightLiteral& p, uint32& undoPos) {
+uint32 MinimizeConstraint::computeImplicationSet(const Solver& s, const WeightLiteral& p, uint32& undoPos) {
 	// start from current sum
 	STRATEGY(assign(temp_, sum_));
 	uint32 up       = undoTop_;
@@ -235,68 +234,57 @@ Constraint* MinimizeConstraint::cloneAttach(Solver&) {
 
 // Stores the current model as the optimum and determines the decision level 
 // on which the search should continue.
-// Returns DL(p)-1, where
-// p is the literal from this constraint that was assigned true last
-uint32 MinimizeConstraint::setModel(Solver& s) {
-	shared_->setOptimum(STRATEGY(sumToOpt()));
+uint32 MinimizeConstraint::commitCurrent(const Solver& s) {
+	// store current sum as new optimum in shared...
+	const Optimum* opt = shared_->setOptimum(STRATEGY(sumToOpt()));
+	// ...and *this
+	std::memcpy(tempOpt_, sum_, numRules()*sizeof(wsum_t));
+	uint32 dl = s.decisionLevel();
 	if (shared_->mode() == MinimizeMode_t::optimize) {
-		// After updating the optimum we have that sum > opt, i.e. the constraint
-		// is conflicting under the current assignment. Hence, we must at least
-		// undo all levels including cfl.
-		return updateOpt(&s, true, true) - 1;
+		// temporarily assume next tentative optimum and
+		// compute first level on which constraint is violated
+		dl = updateOpt(&s, true, opt);
 	}
-	return (uint32)s.decisionLevel()-1;	
-}
-
-// propagates a newly set optimum
-bool MinimizeConstraint::propagateNewOptimum(Solver& s) {
-	assert(!s.hasConflict());
-	// Since we updated the optimum we must re-evaluate all literals
-	pos_    = shared_->lits;
-	do {
-		if (propagateImpl(s, propagate_new_opt)) {
-			return true;
-		}
-		assert(s.hasConflict());
-	} while (s.resolveConflict());
-	return false;
+	std::swap(tempOpt_, opt_);
+	return dl - 1;
 }
 
 // resets the optimum to the one stored in the shared data object
 bool MinimizeConstraint::restoreOptimum() {
 	pos_ = shared_->lits;
-	updateOpt(0, false, false);
+	updateOpt(0, false, shared_->optimum());
 	return opt_[0] != std::numeric_limits<wsum_t>::max();
 }
 
-// integrates a new optimum that was not set by this constraint
+// integrates a new tentative optimum
 bool MinimizeConstraint::integrateNext(Solver& s) {
-	assert(!s.hasConflict() && s.queueSize() == 0);
+	assert(!s.hasConflict() && s.isTrue(s.sharedContext()->tagLiteral()));
 	std::swap(tempOpt_, opt_);
-	uint32 dl = updateOpt(&s, true, false);
-	if (dl == 0) {
-		std::swap(tempOpt_, opt_);
-		s.setStopConflict();
-		return false;
-	}
-	else if (dl != UINT32_MAX && s.undoUntil(dl-1, true) != (dl-1)) {
-		// we could not undo all relevant levels - 
-		// force conflict.
-		std::swap(tempOpt_, opt_);
-		return s.force(negLit(0), this, undoTop_);
-	}
-	return propagateNewOptimum(s);
+	uint32 cl = updateOpt(&s, true, shared_->optimum());
+	do {
+		pos_    = shared_->lits;
+		if (cl > s.decisionLevel() && propagateImpl(s, propagate_new_opt)) {
+			return true;
+		}
+		if (cl <= s.decisionLevel()&& s.undoUntil(cl-1, true) != cl-1) {
+			s.undoUntil(cl, true);
+			if (cl != s.rootLevel()) { s.force(negLit(0), this, undoTop_); }
+			else                     { s.setStopConflict(); }
+		}
+	} while (!s.hasConflict() || s.resolveConflict());
+	// integration failed - restore current optimum
+	std::swap(tempOpt_, opt_);
+	return false;
 }
 
 bool MinimizeConstraint::modelHeuristic(Solver& s) {
-	if (s.propagate()) {
-		const bool heuristic = modelHeu_;
-		for (const WeightLiteral* w = shared_->lits; !isSentinel(w->first); ++w) {
+	const bool modelHeu  = (s.strategies().heuOpts & SolverStrategies::heu_use_model) != 0;
+	const bool heuristic = modelHeu || (s.queueSize() == 0 && s.decisionLevel() == s.rootLevel());
+	if (heuristic && s.propagate()) {
+		for (const WeightLiteral* w = pos_; !isSentinel(w->first); ++w) {
 			if (s.value(w->first.var()) == value_free) {
 				s.assume(~w->first);
-				if (!heuristic || !s.propagate()) {
-					break;
-				}
+				if (!modelHeu || !s.propagate()) { break; }
 			}
 		}
 	}
@@ -306,24 +294,24 @@ bool MinimizeConstraint::modelHeuristic(Solver& s) {
 // Sets (opt-applyStep?step:0) as the new optimum and returns the numerical
 // lowest decision level on which the constraint is conflicting w.r.t
 // the new optimum or UINT32_MAX if the constraint is not conflicting.
-uint32 MinimizeConstraint::updateOpt(Solver* s, bool applyStep, bool model) {
+uint32 MinimizeConstraint::updateOpt(const Solver* s, bool applyStep, const Optimum* opt) {
 	assert(s != 0 || !applyStep);
 	active_[0] = active_[1] = 0;
 	uint32 seq;
 	bool   ret;
-	SharedData::Optimum::Step step;
-	for (const SharedData::Optimum* opt;;) {
-		opt = shared_->optimum();
+	Optimum::Step step;
+	for (;;) {
 		seq = opt->seq;
-		ret = shared_->convert(opt, applyStep, opt_);
-		step= opt->step;
-		if (seq == opt->seq) {
-			break;
-		}
+ 		ret = shared_->convert(opt, applyStep, opt_);
+ 		step= opt->step;
+ 		if (seq == opt->seq) {
+ 			break;
+ 		}
+		// optimum has changed - reload
+		opt = shared_->optimum();
 	}
 	if (!ret) return 0;
-	uint32 len = STRATEGY(convert(opt_));
-	if (model || (applyStep && greater(sum_, opt_, len))) {
+	if (applyStep && greater(sum_, opt_, STRATEGY(convert(opt_)))) {
 		assert(s);
 		uint32 lev = step.first;
 		if (lev == shared_->maxLevel() && step.second == 1 && (sum_[lev]-opt_[lev]) == 1) {
@@ -368,7 +356,8 @@ void MinimizeConstraint::subtract(wsum_t* lhs, weight_t wOrIdx) {
 bool MinimizeConstraint::implied(wsum_t* lhs, weight_t wOrIdx) {
 	if (type_ == SINGLE_LEVEL_ARITH) { return (*lhs+wOrIdx) > *opt_; }
 	const SharedData::LevelWeight* w = &shared_->weights[wOrIdx];
-	uint32& a = active(lhs); assert(a <= w->level);
+	uint32& a = active(lhs); 
+	if (w->level < a) { a = w->level; }
 	while (a != w->level) {
 		if (lhs[a] != opt_[a]) {
 			return lhs[a] > opt_[a];
@@ -475,6 +464,7 @@ void SharedMinimizeData::setStep(Optimum* o) {
 
 bool SharedMinimizeData::convert(const Optimum* opt, bool applyStep, wsum_t* optOut) const {
 	if (applyStep && opt->step.second == 0) return false;
+	applyStep &= opt->opt[0] != std::numeric_limits<wsum_t>::max();
 	uint32 i   = static_cast<uint32>(opt->opt.size());
 	uint32 lev = applyStep ? opt->step.first : i;
 	wsum_t n   = opt->step.second;
@@ -537,7 +527,7 @@ void MinimizeBuilder::adjustSum(LitRep lit) {
 
 // merges duplicate literals and removes literals that are already assigned
 // POST: the literals in lits_ are unique and sorted by decreasing weight
-void MinimizeBuilder::prepare(SharedContext& ctx) {
+bool MinimizeBuilder::prepare(SharedContext& ctx) {
 	std::sort(lits_.begin(), lits_.end(), CmpByLit());
 	LitVec::size_type j = 0;
 	Solver& s           = *ctx.master();
@@ -608,6 +598,14 @@ void MinimizeBuilder::prepare(SharedContext& ctx) {
 	}
 	// add terminating sentinel literal
 	lits_.push_back(LitRep(posLit(0), new Weight(static_cast<uint32>(initial_.size()-1), 0)));
+	bool ok = true;
+	for (uint32 i = 0, end = std::min((uint32)initial_.size(), (uint32)opts_.size()); i != end; ++i) {
+		if (initial_[i] != opts_[i]) {
+			ok = initial_[i] < opts_[i];
+			break;
+		}
+	}
+	return ok;
 }
 
 // PRE: the literals in lits_ are unique and sorted by decreasing weight
@@ -625,11 +623,14 @@ MinimizeBuilder::SharedData* MinimizeBuilder::build(SharedContext& ctx, Literal
 	assert(!ctx.master()->hasConflict());
 	if (!ctx.master()->propagate()) return 0;
 	if (!ready_) {
-		prepare(ctx);
+		if (!prepare(ctx)) return 0;
 		addAssumption(ma);
 		ready_ = true;
 	}
 	opts_.resize(initial_.size(), std::numeric_limits<wsum_t>::max());
+	if (opts_.back() != std::numeric_limits<wsum_t>::max()) {
+		opts_.back() += 1;
+	}
 	SharedData* srep = new (::operator new(sizeof(SharedData) + (lits_.size()*sizeof(WeightLiteral)))) SharedData(initial_, opts_);
 	if (initial_.size() == 1) {
 		for (LitVec::size_type i = 0; i != lits_.size(); ++i) {
@@ -659,16 +660,18 @@ MinimizeBuilder::SharedData* MinimizeBuilder::build(SharedContext& ctx, Literal
 	return srep;
 }
 
-MinimizeConstraint* MinimizeBuilder::buildAndAttach(SharedContext& ctx, MinimizeMode mode, int heuristic) {
-	SharedData* d         = build(ctx);
-	d->setMode(mode, false);
-	MinimizeConstraint* m = attach(*ctx.master(), d, heuristic);
-	m->ownsShared_ = true;
-	return m;
+MinimizeConstraint* MinimizeBuilder::buildAndAttach(SharedContext& ctx, MinimizeMode mode) {
+	if (SharedData* d = build(ctx)) {
+		d->setMode(mode, false);
+		MinimizeConstraint* m = attach(*ctx.master(), d);
+		m->ownsShared_ = true;
+		return m;
+	}
+	return 0;
 }
 
-MinimizeConstraint* MinimizeBuilder::attach(Solver& s, SharedData* data, int heuristic) {
-	MinimizeConstraint* ret = new MinimizeConstraint(data, (unsigned)std::min(std::max(0, heuristic), 3));
+MinimizeConstraint* MinimizeBuilder::attach(Solver& s, SharedData* data) {
+	MinimizeConstraint* ret = new MinimizeConstraint(data);
 	ret->attach(s);
 	return ret;
 }
