@@ -96,11 +96,11 @@ BacktrackEnumerator::BacktrackEnumerator(uint32 opts, Report* p)
 }
 
 
-Enumerator::SolveConstraint* BacktrackEnumerator::doInit(SharedContext& ctx, uint32, bool start) {
+Enumerator::EnumeratorConstraint* BacktrackEnumerator::doInit(SharedContext& ctx, uint32 t, bool start) {
 	if (start) {
 		ModelEnumerator::initContext(ctx);
 	}
-	else if (projectionEnabled()) {
+	else if (projectionEnabled() || t > 1) {
 		return new LocalConstraint();
 	}
 	return 0;
@@ -130,6 +130,8 @@ uint32 BacktrackEnumerator::getHighestBacktrackLevel(const Solver& s, uint32 bl)
 	return res;
 }
 
+bool BacktrackEnumerator::updateModel(Solver&) {  return projectionEnabled() || enumerated == 0; }
+
 bool BacktrackEnumerator::doBacktrack(Solver& s, uint32 bl) {
 	// bl is the decision level on which search should proceed.
 	// bl + 1 the minimum of:
@@ -225,7 +227,7 @@ void BacktrackEnumerator::LocalConstraint::destroy(Solver* s, bool detach) {
 		nogoods.pop_back();
 		c->destroy(s, detach);
 	}
-	SolveConstraint::destroy(s, detach);
+	EnumeratorConstraint::destroy(s, detach);
 }
 void BacktrackEnumerator::LocalConstraint::add(Solver& s, LearntConstraint* c) {
 	if (s.decisionLevel() != 0) {
@@ -245,7 +247,7 @@ void BacktrackEnumerator::LocalConstraint::undoLevel(Solver& s) {
 
 Constraint* BacktrackEnumerator::LocalConstraint::cloneAttach(Solver& other) {
 	LocalConstraint* ret = new LocalConstraint();
-	ret->setMinimize(cloneMini(other));
+	ret->attach(other);
 	return ret;
 }
 
@@ -271,7 +273,7 @@ void RecordEnumerator::SolutionQueue::releaseLits(SL* x) {
 /////////////////////////////////////////////////////////////////////////////////////////
 // class RecordEnumerator::RecordConstraint
 /////////////////////////////////////////////////////////////////////////////////////////
-class RecordEnumerator::LocalConstraint : public SolveConstraint {
+class RecordEnumerator::LocalConstraint : public EnumeratorConstraint {
 public:
 	typedef SolutionQueue::ThreadId ThreadId;
 	LocalConstraint(SolutionQueue* q);
@@ -302,12 +304,12 @@ void RecordEnumerator::LocalConstraint::destroy(Solver* s, bool detach) {
 		db.back()->destroy(s, detach);
 		db.pop_back();
 	}
-	SolveConstraint::destroy(s, detach);
+	EnumeratorConstraint::destroy(s, detach);
 }
 
 Constraint* RecordEnumerator::LocalConstraint::cloneAttach(Solver& other) {
 	LocalConstraint* ret = new LocalConstraint(queue);
-	ret->setMinimize(cloneMini(other));
+	ret->attach(other);
 	return ret;
 }
 
@@ -345,6 +347,7 @@ bool RecordEnumerator::LocalConstraint::integrate(Solver& s, SL* clause, uint32
 	}
 	return res.ok;
 }
+
 /////////////////////////////////////////////////////////////////////////////////////////
 // class RecordEnumerator
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -357,7 +360,7 @@ RecordEnumerator::~RecordEnumerator() {
 	delete queue_;
 }
 
-Enumerator::SolveConstraint* RecordEnumerator::doInit(SharedContext& ctx, uint32 t, bool start) {
+Enumerator::EnumeratorConstraint* RecordEnumerator::doInit(SharedContext& ctx, uint32 t, bool start) {
 	if (start) {
 		ModelEnumerator::initContext(ctx);
 		return 0;
@@ -411,6 +414,9 @@ bool RecordEnumerator::doBacktrack(Solver& s, uint32 bl) {
 	return c->add(s, solution_.lits(), ClauseInfo().setType(Constraint_t::learnt_other));
 }
 
+bool RecordEnumerator::updateModel(Solver&) { 
+	return queue_ != 0;
+}
 bool RecordEnumerator::updateConstraint(Solver& s, bool) {
 	LocalConstraint* con = static_cast<LocalConstraint*>(s.getEnumerationConstraint());
 	return con->integrateAll(s);
