@@ -396,16 +396,20 @@ void ParallelSolve::startSolve(Solver& s, const SolveParams& p, const LitVec& as
 	}
 }
 
+bool ParallelSolve::initOpt(Solver& s, ValueRep last) {
+	return last != value_true
+	  ||   shared_->enumerator()->continueFromModel(s, false);
+}
 // main solve loop executed by all threads
 bool ParallelSolve::doSolve(Solver& s, const SolveParams& p, const LitVec& assume) {
 	PathPtr a(0); InitParams init = p.init;
 	try {
 		startSolve(s, p, assume);
-		for (; requestWork(s, a);) {
+		for (ValueRep last = value_free; requestWork(s, a);) {
 			thread_[s.id()]->prepareForGP(*a, shared_->maxConflict, a.is_owner());
 			s.stats.reset();
-			if (initPath(s, *a, init)) {
-				if (solvePath(s, p) == value_free) {
+			if (initOpt(s, last) && initPath(s, *a, init)) {
+				if ((last = solvePath(s, p)) == value_free) {
 					terminate(s, false);
 				}
 				s.clearStopConflict();
@@ -487,7 +491,7 @@ bool ParallelSolve::terminate() {
 	// deadlock in semaphore!
 	shared_->postMessage(SharedData::msg_interrupt, false);
 	// notify any thread currently in Enumerator::backtrackFromModel()
-	shared_->ctx->enumerator()->terminate();
+	shared_->enumerator()->terminate();
 	return true;
 }
 
@@ -657,11 +661,11 @@ bool ParallelSolve::backtrackFromModel(Solver& s) {
 			shared_->setControl(SharedData::forbid_restart_flag | SharedData::allow_gp_flag);
 		}
 	}
-	if (r == Enumerator::enumerate_continue) { 
-		// continue only after we released the mutex to avoid deadlock -
-		// continueFromModel(s) might call s.propagate() which could
-		// block on a sync message.
-		r = shared_->enumerator()->continueFromModel(s); 
+	// continue only after we released the mutex to avoid deadlock -
+	// continueFromModel(s) might call s.propagate() which could
+	// block on a sync message.
+	if (!shared_->enumerator()->continueFromModel(s)) {
+		r = Enumerator::enumerate_stop_complete;
 	}
 	return r == Enumerator::enumerate_continue && !shared_->terminate();
 }
