@@ -18,7 +18,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifdef _MSC_VER
+#if defined(_MSC_VER)
 #pragma warning (disable : 4146) // unary minus operator applied to unsigned type, result still unsigned
 #pragma warning (disable : 4996) // 'std::_Fill_n' was declared deprecated
 #endif
@@ -33,6 +33,26 @@
 #include <sstream>
 #include <numeric>
 
+#if defined(_MSC_VER)
+#	if !defined(__FUNCTION__)
+#	define MY_FUNCNAME __FILE__
+#	else
+# define MY_FUNCNAME __FUNCTION__
+# endif
+#elif defined(__GNUC__)
+#define MY_FUNCNAME __PRETTY_FUNCTION__
+#else
+#define MY_FUNCNAME __FILE__
+#endif
+
+std::string precondition_error(const char* ex, const char* func, unsigned line) {
+	std::stringstream err;
+	err << func << "@" << line << ": precondition violated: " << ex;
+	return err.str();
+}
+
+#define check_precondition(x, EX) \
+	(void)( (!!(x)) || (throw EX(precondition_error((#x), MY_FUNCNAME, __LINE__)), 0))
 
 namespace Clasp {
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -156,7 +176,7 @@ PrgBodyNode::PrgBodyNode(uint32 id, const PrgRule& rule, const PrgRule::RData& r
 			goals_[p] = x;
 			a->posDep.push_back(id);
 			if (w) { 
-				assert(rule.body[i].second>0);
+				check_precondition(rule.body[i].second>0, std::logic_error);
 				spw += (extra_.ext->weights[p] = rule.body[i].second);
 			}
 			++p;
@@ -165,7 +185,7 @@ PrgBodyNode::PrgBodyNode(uint32 id, const PrgRule& rule, const PrgRule::RData& r
 			goals_[posSize_+n]  = x;
 			if (prg.eqOpts_.iters != 0) { a->negDep.push_back(id); }
 			if (w) { 
-				assert(rule.body[i].second>0);
+				check_precondition(rule.body[i].second>0, std::logic_error);
 				snw += (extra_.ext->weights[posSize_+n] = rule.body[i].second);
 			}
 			++n;
@@ -802,11 +822,14 @@ void PreproStats::accu(const PreproStats& o) {
 /////////////////////////////////////////////////////////////////////////////////////////
 // class ProgramBuilder
 /////////////////////////////////////////////////////////////////////////////////////////
-ProgramBuilder::ProgramBuilder() : minimize_(0), incData_(0), graph_(0) { }
+ProgramBuilder::ProgramBuilder() 
+	: minimize_(0), incData_(0), graph_(0), frozen_(true) { 
+}
 ProgramBuilder::~ProgramBuilder() { disposeProgram(true); }
 ProgramBuilder::Incremental::Incremental() : startAtom_(1), startVar_(1), startAux_(1), startScc_(0) {}
 
 void ProgramBuilder::disposeProgram(bool force) {
+	// remove rules
 	std::for_each( bodies_.begin(), bodies_.end(), DestroyObject() );
 	BodyList().swap(bodies_);
 	bodyIndex_.clear();
@@ -832,39 +855,35 @@ void ProgramBuilder::disposeProgram(bool force) {
 		vars_.clear();
 		ruleState_.clear();
 	}
-	stats.reset();
-}
-
-ProgramBuilder& ProgramBuilder::setAtomName(Var atomId, const char* name) {
-	resize(atomId);
-	ctx_->symTab().addUnique(atomId, name);
-	return *this;
-}
-
-ProgramBuilder& ProgramBuilder::setCompute(Var atomId, bool pos) {
-	resize(atomId);
-	ValueRep v = pos ? value_weak_true : value_false;
-	compute_.push_back(Literal(atomId, v==value_false));
-	if (!atoms_[atomId]->setValue(v)) {
-		setConflict();
+	else {
+		// clean up atoms
+		// remove rule associations
+		uint32 startAux = incData_ ? incData_->startAux_ : (uint32)atoms_.size();
+		assert(startAux <= atoms_.size());
+		for (VarVec::size_type i = 0; i != startAux; ++i) {
+			if (atoms_[i]->eq() && getEqAtom(i) >= startAux) {
+				// atom i is equivalent to some aux atom 
+				// make i the new root
+				PrgAtomNode* eq = atoms_[getEqAtom(i)];
+				assert(!eq->eq());
+				eq->posDep.clear(); eq->negDep.clear(); eq->preds.clear();
+				atoms_[i]->assign(eq);
+				eq->setEq(i);
+			}
+			VarVec().swap(atoms_[i]->posDep);
+			VarVec().swap(atoms_[i]->negDep);
+			HeadVec().swap(atoms_[i]->preds);
+		}
+		// delete any introduced aux atoms
+		// this is safe because aux atoms are never part of the input program
+		// it is necessary in order to free their ids, i.e. the id of an aux atom
+		// from step I might be needed for a program atom in step I+1
+		for (VarVec::size_type i = startAux; i != atoms_.size(); ++i) {
+			delete atoms_[i];
+		}
+		atoms_.erase(atoms_.begin()+startAux, atoms_.end());
 	}
-	return *this;
-}
-
-ProgramBuilder& ProgramBuilder::freeze(Var atomId) {
-	PrgAtomNode* a = resize(atomId);
-	a->setFrozen(true);
-	if (!incData_) incData_ = new Incremental();
-	incData_->freeze_.push_back(atomId);
-	return *this;
-}
-
-ProgramBuilder& ProgramBuilder::unfreeze(Var atomId) {
-	resize(atomId);
-	assert(atoms_[atomId]->frozen() && "Can't unfreeze non-frozen atom!");
-	if (!incData_) incData_ = new Incremental();
-	incData_->unfreeze_.push_back(atomId);
-	return *this;
+	stats.reset();
 }
 
 ProgramBuilder& ProgramBuilder::startProgram(SharedContext& ctx, const EqOptions& eqOpts) {
@@ -885,51 +904,187 @@ ProgramBuilder& ProgramBuilder::startProgram(SharedContext& ctx, const EqOptions
 }
 
 ProgramBuilder& ProgramBuilder::updateProgram() {
-	if (!incData_)  { incData_ = new Incremental(); }
-	else            {
-		uint32 maxVar = ctx_->master() ? ctx_->master()->numVars() : 1;
-		while ((uint32)vars_.size() <= maxVar) {
-			vars_.add(Var_t::atom_var);
-		}
-		incData_->startVar_   = maxVar + 1;
-		incData_->unfreeze_.clear();
-		for (VarVec::iterator it = incData_->freeze_.begin(), end = incData_->freeze_.end(); it != end; ++it) {
-			atoms_[*it]->setIgnore(false);
-		}
-	}
-	// delete bodies...
+	check_precondition(frozen_ || !incData_, std::logic_error);
+	check_precondition(!atoms_.empty() && "startProgram() not called!", std::logic_error);
+	// delete bodies and clean up atoms
 	disposeProgram(false);
-	// clean up atoms
-	assert(incData_->startAux_ <= atoms_.size());
-	for (VarVec::size_type i = 0; i != incData_->startAux_; ++i) {
-		if (atoms_[i]->eq() && getEqAtom(i) >= incData_->startAux_) {
-			// atom i is equivalent to some aux atom 
-			// make i the new root
-			PrgAtomNode* eq = atoms_[getEqAtom(i)];
-			assert(!eq->eq());
-			eq->posDep.clear(); eq->negDep.clear(); eq->preds.clear();
-			atoms_[i]->assign(eq);
-			eq->setEq(i);
-		}
-		VarVec().swap(atoms_[i]->posDep);
-		VarVec().swap(atoms_[i]->negDep);
-		HeadVec().swap(atoms_[i]->preds);
+	if (!incData_)  { incData_ = new Incremental(); }
+	uint32 maxVar = ctx_->master() ? ctx_->master()->numVars() : 1;
+	while ((uint32)vars_.size() <= maxVar) {
+		vars_.add(Var_t::atom_var);
 	}
-	// delete any introduced aux atoms
-	// this is safe because aux atoms are never part of the input program
-	// it is necessary in order to free their ids, i.e. the id of an aux atom
-	// from step I might be needed for a program atom in step I+1
-	for (VarVec::size_type i = incData_->startAux_; i != atoms_.size(); ++i) {
-		delete atoms_[i];
+	incData_->startVar_ = (uint32)vars_.size();
+	incData_->startAtom_= (uint32)atoms_.size();
+	incData_->startAux_ = (uint32)atoms_.size();
+	incData_->unfreeze_.clear();
+	for (VarVec::iterator it = incData_->freeze_.begin(), end = incData_->freeze_.end(); it != end; ++it) {
+		atoms_[*it]->setIgnore(false);
 	}
-	atoms_.erase(atoms_.begin()+incData_->startAux_, atoms_.end());
-	incData_->startAtom_  = (uint32)atoms_.size();
-	incData_->startAux_   = (uint32)atoms_.size();
 	frozen_   = false;
 	ctx_->symTab().startInit();
 	return *this;
 }
 
+bool ProgramBuilder::endProgram() {
+	return frozen_ || (prepareProgram() && transformProgram(!eqOpts_.noSCC));
+}
+
+bool ProgramBuilder::cloneProgram(SharedContext& ctx) {
+	assert(frozen_);
+	if (&ctx == ctx_) {
+		return true;
+	}
+	if (&ctx.symTab() != &ctx_->symTab()) {
+		ctx_->symTab().copyTo(ctx.symTab());
+	}
+	vars_.addTo(ctx, incData_ ? incData_->startVar_ : 1);
+	SharedContext* t = ctx_;
+	ctx_   = &ctx;
+	bool r = transformProgram(false);
+	ctx_   = t;
+	return r;
+}
+
+void ProgramBuilder::addMinimize(MinimizeBuilder& m) {
+	check_precondition(frozen_, std::length_error);
+	if (minimize_) {
+		WeightLitVec lits;
+		for (MinimizeRule* r = minimize_; r; r = r->next_) {
+			for (WeightLitVec::iterator it = r->lits_.begin(); it != r->lits_.end(); ++it) {
+				PrgAtomNode* h    = atoms_[it->first.var()];
+				lits.push_back(WeightLiteral(it->first.sign() ? ~h->literal() : h->literal(), it->second));
+			}
+			m.addRule(lits);
+			lits.clear();
+		}
+	}
+}
+
+void ProgramBuilder::writeProgram(std::ostream& os) {
+	const char* const delimiter = "0";
+	// first write all minimize rules - revert order!
+	PodVector<MinimizeRule*>::type mr;
+	for (MinimizeRule* r = minimize_; r; r = r->next_) {
+		mr.push_back(r);
+	}
+	for (PodVector<MinimizeRule*>::type::reverse_iterator rit = mr.rbegin(); rit != mr.rend(); ++rit) {
+		os << OPTIMIZERULE << " " << 0 << " ";
+		std::stringstream pBody, nBody, weights;
+		VarVec::size_type nbs = 0, pbs =0;
+		MinimizeRule* r = *rit;
+		for (WeightLitVec::iterator it = r->lits_.begin(); it != r->lits_.end(); ++it) {
+			if (atoms_[it->first.var()]->hasVar()) {
+				it->first.sign() ? ++nbs : ++pbs;
+				std::stringstream& body = it->first.sign() ? nBody : pBody;
+				body << it->first.var() << " ";
+				weights << it->second << " ";
+			}
+		}
+		os << pbs+nbs << " " << nbs << " "
+			 << nBody.str() << pBody.str() << weights.str() << "\n";
+	}
+	uint32 falseAtom = 0;
+	for (uint32 i = 1; i < atoms_.size(); ++i) {
+		if (atoms_[i]->value() == value_false) {
+			falseAtom = i;
+			break;
+		}
+	}
+	// write all bodies together with their heads
+	for (BodyList::iterator it = bodies_.begin(); it != bodies_.end(); ++it) {
+		PrgBodyNode* b = *it;
+		if (b->hasVar()) {
+			if (b->value() == value_false && !b->hasHeads() && !b->ignore()) {
+				// This body is an integrity constraint.
+				// Handle by writing falseAtom :- Body.
+				// where falseAtom is set to false in the compute statement
+				if (falseAtom == 0) {
+					atoms_.push_back( new PrgAtomNode() );
+					falseAtom = (Var) atoms_.size() - 1;
+					setCompute(falseAtom, false);
+				}
+				b->addHead(falseAtom, BASICRULE);
+			}
+			writeRule(b, falseAtom, os);
+		}
+	}
+	// write eq-atoms, symbol-table and compute statement
+	std::stringstream bp, bm, symTab;
+	Literal comp;
+	SymbolTable::const_iterator sym = ctx_->symTab().begin();
+	for (AtomList::size_type i = 1; i < atoms_.size(); ++i) {
+		// write the equivalent atoms
+		if (atoms_[i]->eq()) {
+			os << "1 " << i << " 1 0 " << getEqAtom(Var(i)) << " \n";
+		}
+		if ( atoms_[i]->value() != value_free ) {
+			std::stringstream& str = atoms_[i]->value() == value_false ? bm : bp;
+			str << i << "\n";
+		}
+		if (sym != ctx_->symTab().end() && Var(i) == sym->first) {
+			if (sym->second.lit != negLit(sentVar) && !sym->second.name.empty()) {
+				symTab << i << " " << sym->second.name.c_str() << "\n";
+			}
+			++sym;
+		}
+	}
+	os << delimiter << "\n";
+	os << symTab.str();
+	os << delimiter << "\n";
+	os << "B+\n" << bp.str() << "0\n"
+		 << "B-\n" << bm.str() << "0\n1\n";
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////
+// Program mutating functions
+/////////////////////////////////////////////////////////////////////////////////////////
+#define check_not_frozen() check_precondition(!frozen_ && "Can't update frozen program!", std::logic_error)
+Var ProgramBuilder::newAtom() {
+	check_not_frozen();
+	atoms_.push_back( new PrgAtomNode() );
+	return (Var) atoms_.size() - 1;
+}
+
+ProgramBuilder& ProgramBuilder::setAtomName(Var atomId, const char* name) {
+	check_not_frozen();
+	check_precondition(atomId >= startAtom(), RedefinitionError);
+	resize(atomId);
+	ctx_->symTab().addUnique(atomId, name);
+	return *this;
+}
+
+ProgramBuilder& ProgramBuilder::setCompute(Var atomId, bool pos) {
+	PrgAtomNode* a = resize(atomId);
+	ValueRep v     = pos ? value_weak_true : value_false;
+	compute_.push_back(Literal(atomId, v==value_false));
+	if (!a->setValue(v)) {
+		setConflict();
+	}
+	return *this;
+}
+
+ProgramBuilder& ProgramBuilder::freeze(Var atomId) {
+	check_not_frozen();
+	check_precondition(incData_ && "ProgramBuilder::updateProgram() not called!", std::logic_error);
+	PrgAtomNode* a = resize(atomId);
+	if (atomId >= startAtom() && !a->frozen() && a->preds.empty()) {
+		incData_->freeze_.push_back(atomId);
+		a->setFrozen(true);
+	}
+	// else: atom is defined or from a previous step - ignore!
+	return *this;
+}
+
+ProgramBuilder& ProgramBuilder::unfreeze(Var atomId) {
+	check_not_frozen();
+	check_precondition(incData_ && "ProgramBuilder::updateProgram() not called!", std::logic_error);
+	PrgAtomNode* atom = resize(atomId);
+	if (atomId >= startAtom() || atom->frozen()) {
+		incData_->unfreeze_.push_back(atomId);
+	}
+	// else: atom is from a previous step - ignore!
+	return *this;
+}
 
 ProgramBuilder& ProgramBuilder::addRule(PrgRule& r) {
 	// simplify rule, mark literals
@@ -938,6 +1093,7 @@ ProgramBuilder& ProgramBuilder::addRule(PrgRule& r) {
 	}
 	PrgRule::RData rd = r.simplify(ruleState_);
 	if (r.type() != ENDRULE) {     // rule is relevant
+		check_not_frozen();
 		upRules(r.type(), 1);
 		if (handleNatively(r, rd)) { // and can be handled natively
 			addRuleImpl(r, rd);
@@ -963,6 +1119,62 @@ ProgramBuilder& ProgramBuilder::addRule(PrgRule& r) {
 	}
 	return *this;
 }
+#undef check_not_frozen
+/////////////////////////////////////////////////////////////////////////////////////////
+// Query functions
+/////////////////////////////////////////////////////////////////////////////////////////
+Literal ProgramBuilder::getLiteral(Var atomId) const {
+	check_precondition(atomId < atoms_.size(), std::logic_error);
+	return getAtom(getEqAtom(atomId))->literal();
+}
+void ProgramBuilder::getAssumptions(LitVec& out) const {
+	check_precondition(frozen_, std::logic_error);
+	if (incData_) {
+		for (VarVec::const_iterator it = incData_->freeze_.begin(), end = incData_->freeze_.end(); it != end; ++it) {
+			out.push_back( ~getLiteral(*it) );
+		}
+	}
+}
+/////////////////////////////////////////////////////////////////////////////////////////
+// Program definition - private
+/////////////////////////////////////////////////////////////////////////////////////////
+bool ProgramBuilder::inCompute(Literal x) const {
+	return std::find(compute_.begin(), compute_.end(), x) != compute_.end();
+}
+void ProgramBuilder::addRuleImpl(PrgRule& r, const PrgRule::RData& rd) {
+	if (r.type() != OPTIMIZERULE) {
+		Body b = findOrCreateBody(r, rd);
+		if (b.first->value() == value_false || rd.value == value_false) {
+			// a false body can't define any atoms
+			b.first->propagateFalse(b.second, atoms_);
+		}
+		for (VarVec::const_iterator it = r.heads.begin(), end = r.heads.end(); it != end; ++it) {
+			if (b.first->value() != value_false) {
+				PrgAtomNode* a = resize(*it);
+				check_precondition(*it >= startAtom() || a->frozen() || inCompute(negLit(*it)), RedefinitionError);
+				if (r.body.empty())                  { a->setIgnore(true); }
+				if (a->frozen() && a->preds.empty()) { unfreeze(*it); }
+				// Note: b->heads may now contain duplicates. They are removed in PrgBodyNode::buildHeadSet.
+				HeadType t = b.first->addHead(*it, r.type());
+				// Similarly, duplicates in atoms_[*it]->preds_ are removed in PrgAtomNode::simplifyBodies.
+				a->preds.push_back(HeadEdge(b.second, t));
+			}
+			ruleState_.popFromRule(*it);  // clear flag of head atoms
+		}
+		if (rd.value != value_free) { b.first->setValue(rd.value); }
+	}
+	else {
+		check_precondition(r.heads.empty(), std::logic_error);
+		ProgramBuilder::MinimizeRule* mr = new ProgramBuilder::MinimizeRule;
+		for (WeightLitVec::const_iterator it = r.body.begin(), bEnd = r.body.end(); it != bEnd; ++it) {
+			resize(it->first.var());
+			ruleState_.popFromRule(it->first.var());
+		}
+		mr->lits_ = r.body;
+		mr->next_ = minimize_;
+		minimize_ = mr;
+	}
+}
 
 void ProgramBuilder::updateRule(PrgRule& r) {
 	for (WeightLitVec::iterator it = r.body.begin(), end = r.body.end(); it != end; ++it) {
@@ -998,42 +1210,6 @@ bool ProgramBuilder::handleNatively(const PrgRule& r, const PrgRule::RData& rd)
 	return true;
 }
 
-void ProgramBuilder::addRuleImpl(PrgRule& r, const PrgRule::RData& rd) {
-	assert(!frozen_ && "Can't add rules to frozen program!");
-	assert(r.type() != ENDRULE);
-	if (r.type() != OPTIMIZERULE) {
-		Body b = findOrCreateBody(r, rd);
-		if (b.first->value() == value_false || rd.value == value_false) {
-			// a false body can't define any atoms
-			b.first->propagateFalse(b.second, atoms_);
-		}
-		for (VarVec::const_iterator it = r.heads.begin(), end = r.heads.end(); it != end; ++it) {
-			if (b.first->value() != value_false) {
-				PrgAtomNode* a = resize(*it);
-				assert( (!a->hasVar() || a->frozen()) && "Cannot extend definition of atom!");
-				if (r.body.empty()) a->setIgnore(true);
-				// Note: b->heads may now contain duplicates. They are removed in PrgBodyNode::buildHeadSet.
-				HeadType t = b.first->addHead(*it, r.type());
-				// Similarly, duplicates in atoms_[*it]->preds_ are removed in PrgAtomNode::simplifyBodies.
-				a->preds.push_back(HeadEdge(b.second, t));
-			}
-			ruleState_.popFromRule(*it);  // clear flag of head atoms
-		}
-		if (rd.value != value_free) { b.first->setValue(rd.value); }
-	}
-	else {
-		assert(r.heads.empty());
-		ProgramBuilder::MinimizeRule* mr = new ProgramBuilder::MinimizeRule;
-		for (WeightLitVec::const_iterator it = r.body.begin(), bEnd = r.body.end(); it != bEnd; ++it) {
-			resize(it->first.var());
-			ruleState_.popFromRule(it->first.var());
-		}
-		mr->lits_ = r.body;
-		mr->next_ = minimize_;
-		minimize_ = mr;
-	}
-}
-
 ProgramBuilder::Body ProgramBuilder::findOrCreateBody(PrgRule& r, const PrgRule::RData& rd) {
 	ProgramBuilder::BodyRange eqRange = bodyIndex_.equal_range(rd.hash);
 	for (bool sorted = false; eqRange.first != eqRange.second; ++eqRange.first) {
@@ -1068,33 +1244,6 @@ ProgramBuilder::Body ProgramBuilder::findOrCreateBody(PrgRule& r, const PrgRule:
 	return Body(b, bodyId);
 }
 
-// checks whether all literals in body are currently marked in the
-// ProgramBuilder's ruleState-Object. Used for comparing bodies
-// in a "mark-and-test"-approach
-bool ProgramBuilder::allLitsMarked(const PrgBodyNode& body) {
-	for (uint32 i = 0, end = body.size(); i != end; ++i) {
-		if (!ruleState_.inBody(body.goal(i))) { return false; }				
-	}
-	return true;
-}
-
-// checks whether body and wl are equal w.r.t literals and weights
-bool ProgramBuilder::eqWeights(const PrgBodyNode& body, WeightLitVec& lits, bool& sorted)  const {
-	compose_2_2<std::less<Literal>, select1st<WeightLiteral>, select1st<WeightLiteral> > cmp;
-	if (!sorted) {
-		std::stable_sort(lits.begin(), lits.end(), cmp);
-		sorted = true;
-	}
-	for (uint32 i = 0, end = body.size(); i != end; ++i) {
-		WeightLiteral wl(body.goal(i), body.weight(i));
-		WeightLitVec::iterator it = std::lower_bound(lits.begin(), lits.end(), wl, cmp);
-		if (it == lits.end() || *it != wl) {
-			return false;
-		}
-	}
-	return body.size() == (uint32)lits.size();
-}
-
 void ProgramBuilder::clearRuleState(const PrgRule& r) {
 	for (VarVec::const_iterator it = r.heads.begin(), end = r.heads.end();  it != end; ++it) {
 		// clear flag only if a node was added for the head!
@@ -1199,6 +1348,33 @@ void ProgramBuilder::transformIntegrity(uint32 maxAux) {
 	}
 }
 
+// checks whether all literals in body are currently marked in the
+// ProgramBuilder's ruleState-Object. Used for comparing bodies
+// in a "mark-and-test"-approach
+bool ProgramBuilder::allLitsMarked(const PrgBodyNode& body) {
+	for (uint32 i = 0, end = body.size(); i != end; ++i) {
+		if (!ruleState_.inBody(body.goal(i))) { return false; }				
+	}
+	return true;
+}
+
+// checks whether body and wl are equal w.r.t literals and weights
+bool ProgramBuilder::eqWeights(const PrgBodyNode& body, WeightLitVec& lits, bool& sorted)  const {
+	compose_2_2<std::less<Literal>, select1st<WeightLiteral>, select1st<WeightLiteral> > cmp;
+	if (!sorted) {
+		std::stable_sort(lits.begin(), lits.end(), cmp);
+		sorted = true;
+	}
+	for (uint32 i = 0, end = body.size(); i != end; ++i) {
+		WeightLiteral wl(body.goal(i), body.weight(i));
+		WeightLitVec::iterator it = std::lower_bound(lits.begin(), lits.end(), wl, cmp);
+		if (it == lits.end() || *it != wl) {
+			return false;
+		}
+	}
+	return body.size() == (uint32)lits.size();
+}
+
 namespace {
 	struct LessBody {
 		bool operator()(PrgBodyNode* lhs, PrgBodyNode* rhs) const {
@@ -1299,26 +1475,6 @@ bool ProgramBuilder::transformProgram(bool checkSccs) {
 	return ret;
 }
 
-bool ProgramBuilder::endProgram() {
-	return frozen_ || (prepareProgram() && transformProgram(!eqOpts_.noSCC));
-}
-
-bool ProgramBuilder::cloneProgram(SharedContext& ctx) {
-	assert(frozen_);
-	if (&ctx == ctx_) {
-		return true;
-	}
-	if (&ctx.symTab() != &ctx_->symTab()) {
-		ctx_->symTab().copyTo(ctx.symTab());
-	}
-	vars_.addTo(ctx, incData_ ? incData_->startVar_ : 1);
-	SharedContext* t = ctx_;
-	ctx_   = &ctx;
-	bool r = transformProgram(false);
-	ctx_   = t;
-	return r;
-}
-
 void ProgramBuilder::updateFrozenAtoms() {
 	const Solver& solver = *ctx_->master();
 	if (incData_ != 0) {
@@ -1336,29 +1492,38 @@ void ProgramBuilder::updateFrozenAtoms() {
 				atoms_[i]->setValue(v);
 			}
 		}
-		ExtendedRuleMode oldMode = eqOpts_.erMode;
-		eqOpts_.erMode           = mode_native;
+		// remove protection of frozen atoms
+		VarVec::iterator j = incData_->unfreeze_.begin();
 		for (VarVec::iterator it = incData_->unfreeze_.begin(), end = incData_->unfreeze_.end(); it != end; ++it) {
 			PrgAtomNode* a = atoms_[*it];
+			if (a->frozen() && *it < startAtom()) {
+				a->resetSccFlags();
+				*j++ = *it;
+			}
 			a->setFrozen(false);
-			a->resetSccFlags();
 		}
-		VarVec::iterator j = incData_->freeze_.begin();
+		incData_->unfreeze_.erase(j, incData_->unfreeze_.end());
+		// add protection for atoms still frozen
+		j = incData_->freeze_.begin();
+		Body emptyBody(0,0);
 		for (VarVec::iterator it = j, end = incData_->freeze_.end(); it != end; ++it) {
 			PrgAtomNode* a = atoms_[*it];
 			if (a->frozen()) {
 				assert(a->preds.empty() && "Can't freeze defined atom!");
+				if (emptyBody.first == 0) { 
+					rule_.clear(); rule_.setType(CHOICERULE);
+					PrgRule::RData rd = {0,0,0,0};
+					emptyBody = findOrCreateBody(rule_, rd); 
+				}
 				// Make atom a choice.
 				// This way, no special handling during preprocessing/nogood creation is necessary
-				// Note: The choice is removed on next call to updateProgram!
-				startRule(CHOICERULE).addHead(*it).endRule();
-				// Note: Don't count this rule - it is only an implementation detail
-				upRules(CHOICERULE, -1);
+				HeadType t = emptyBody.first->addHead(*it, CHOICERULE);
+				a->preds.push_back(HeadEdge(emptyBody.second, t));
+				a->setIgnore(true);
 				*j++ = *it;
 			}
 		}
 		incData_->freeze_.erase(j, incData_->freeze_.end());
-		eqOpts_.erMode = oldMode;
 	}
 }
 
@@ -1393,6 +1558,8 @@ bool ProgramBuilder::addConstraints(CycleChecker& c) {
 	}
 	const bool freezeAtoms = incData_ && ctx_->satPrepro.get() != 0;
 	uint32 start           = startAtom();
+	check_precondition(ctx_->symTab().curBegin() == ctx_->symTab().end() || start <= ctx_->symTab().curBegin()->first,
+		std::logic_error);
 	SymbolTable::const_iterator sym = ctx_->symTab().lower_bound(ctx_->symTab().curBegin(), start);
 	for (AtomList::const_iterator it = atoms_.begin()+start; it != atoms_.end(); ++it) {
 		if ( !(*it)->toConstraint(*ctx_, gc, *this) ) { return false; }
@@ -1408,20 +1575,6 @@ bool ProgramBuilder::addConstraints(CycleChecker& c) {
 	return true;
 }
 
-void ProgramBuilder::addMinimize(MinimizeBuilder& m) {
-	if (minimize_) {
-		WeightLitVec lits;
-		for (MinimizeRule* r = minimize_; r; r = r->next_) {
-			for (WeightLitVec::iterator it = r->lits_.begin(); it != r->lits_.end(); ++it) {
-				PrgAtomNode* h    = atoms_[it->first.var()];
-				lits.push_back(WeightLiteral(it->first.sign() ? ~h->literal() : h->literal(), it->second));
-			}
-			m.addRule(lits);
-			lits.clear();
-		}
-	}
-}
-
 // exclude vars contained in minimize statements from var elimination
 void ProgramBuilder::freezeMinimize() {
 	if (!minimize_) return;
@@ -1441,12 +1594,6 @@ void ProgramBuilder::setConflict() {
 	atoms_[0]->setValue(value_true);
 }
 
-Var ProgramBuilder::newAtom() {
-	assert(!frozen_ && "Can't add atom to frozen program!");
-	atoms_.push_back( new PrgAtomNode() );
-	return (Var) atoms_.size() - 1;
-}
-
 PrgAtomNode* ProgramBuilder::resize(Var atomId) {
 	assert(atomId != varMax && atomId > 0);
 	while (atoms_.size() <= AtomList::size_type(atomId)) {
@@ -1455,81 +1602,6 @@ PrgAtomNode* ProgramBuilder::resize(Var atomId) {
 	return atoms_[atomId];
 }
 
-void ProgramBuilder::writeProgram(std::ostream& os) {
-	const char* const delimiter = "0";
-	// first write all minimize rules - revert order!
-	PodVector<MinimizeRule*>::type mr;
-	for (MinimizeRule* r = minimize_; r; r = r->next_) {
-		mr.push_back(r);
-	}
-	for (PodVector<MinimizeRule*>::type::reverse_iterator rit = mr.rbegin(); rit != mr.rend(); ++rit) {
-		os << OPTIMIZERULE << " " << 0 << " ";
-		std::stringstream pBody, nBody, weights;
-		VarVec::size_type nbs = 0, pbs =0;
-		MinimizeRule* r = *rit;
-		for (WeightLitVec::iterator it = r->lits_.begin(); it != r->lits_.end(); ++it) {
-			if (atoms_[it->first.var()]->hasVar()) {
-				it->first.sign() ? ++nbs : ++pbs;
-				std::stringstream& body = it->first.sign() ? nBody : pBody;
-				body << it->first.var() << " ";
-				weights << it->second << " ";
-			}
-		}
-		os << pbs+nbs << " " << nbs << " "
-			 << nBody.str() << pBody.str() << weights.str() << "\n";
-	}
-	uint32 falseAtom = 0;
-	for (uint32 i = 1; i < atoms_.size(); ++i) {
-		if (atoms_[i]->value() == value_false) {
-			falseAtom = i;
-			break;
-		}
-	}
-	// write all bodies together with their heads
-	for (BodyList::iterator it = bodies_.begin(); it != bodies_.end(); ++it) {
-		PrgBodyNode* b = *it;
-		if (b->hasVar()) {
-			if (b->value() == value_false && !b->hasHeads() && !b->ignore()) {
-				// This body is an integrity constraint.
-				// Handle by writing falseAtom :- Body.
-				// where falseAtom is set to false in the compute statement
-				if (falseAtom == 0) {
-					atoms_.push_back( new PrgAtomNode() );
-					falseAtom = (Var) atoms_.size() - 1;
-					setCompute(falseAtom, false);
-				}
-				b->addHead(falseAtom, BASICRULE);
-			}
-			writeRule(b, falseAtom, os);
-		}
-	}
-	// write eq-atoms, symbol-table and compute statement
-	std::stringstream bp, bm, symTab;
-	Literal comp;
-	SymbolTable::const_iterator sym = ctx_->symTab().begin();
-	for (AtomList::size_type i = 1; i < atoms_.size(); ++i) {
-		// write the equivalent atoms
-		if (atoms_[i]->eq()) {
-			os << "1 " << i << " 1 0 " << getEqAtom(Var(i)) << " \n";
-		}
-		if ( atoms_[i]->value() != value_free ) {
-			std::stringstream& str = atoms_[i]->value() == value_false ? bm : bp;
-			str << i << "\n";
-		}
-		if (sym != ctx_->symTab().end() && Var(i) == sym->first) {
-			if (sym->second.lit != negLit(sentVar) && !sym->second.name.empty()) {
-				symTab << i << " " << sym->second.name.c_str() << "\n";
-			}
-			++sym;
-		}
-	}
-	os << delimiter << "\n";
-	os << symTab.str();
-	os << delimiter << "\n";
-	os << "B+\n" << bp.str() << "0\n"
-		 << "B-\n" << bm.str() << "0\n1\n";
-}
-
 void ProgramBuilder::writeRule(PrgBodyNode* b, Var falseAtom, std::ostream& os) {
 	VarVec::size_type nbs = 0, pbs = 0;
 	std::stringstream body;
