@@ -410,11 +410,11 @@ bool parseDimacsImpl(std::istream& prg, SharedContext& ctx, bool assertPure, Obj
 	bool wcnf = false, partial = false;
 	int w     = 1;
 	int top   = 1;
-	for (uint32 state = 0;ret;) {
+	for (uint32 state = 0;ret;)  {
 		skipAllWhite(in);
-		if      (*in == 'c')             { skipLine(in); }
-		else if (*in == 0)               { break; }
-		else if (state == 1)             { // read clause
+		if      (*in == 'c')       { skipLine(in); }
+		else if (*in == 0 && state){ break; }
+		else if (state == 1)       { // read clause
 			w = 1;
 			if (wcnf && (!in.parseInt(w) || w < 1)) { ERROR("wcnf: clause weight expected!"); }
 			int lit;
