@@ -29,6 +29,7 @@ namespace Clasp {
 namespace {
 	inline bool isBinary(const Literal* imp) { return imp->watched(); }
 }
+
 #ifndef DISABLE_MULTI_THREADING
 struct ShortImplicationsGraph::Block {
 	typedef std::atomic<uint32> atomic_size;
@@ -65,10 +66,10 @@ struct ShortImplicationsGraph::Block {
 };
 
 #define FOR_EACH_LEARNT(x, Y) \
-	if (0) ; else \
 	for (Block* b = (x).learnt; b ; b = b->next) \
 		for (const Literal* Y = b->begin(), *endof = b->end(); Y != endof; Y += 2 - Y->watched())
 
+	
 ShortImplicationsGraph::ImplicationList::~ImplicationList() {
 	clear(true);
 }
@@ -137,7 +138,7 @@ void ShortImplicationsGraph::ImplicationList::move(ImplicationList& other) {
 	other.learnt = 0;
 }
 #else
-#define FOR_EACH_LEARNT(x, Y) if (const Literal* Y = 0) ; else while(0)
+#define FOR_EACH_LEARNT(x, Y) while (const Literal* Y = 0) 
 #endif
 /////////////////////////////////////////////////////////////////////////////////////////
 // ShortImplicationsGraph
