@@ -191,23 +191,18 @@ ValueRep SolveAlgorithm::solvePath(Solver& s, const SolveParams& p) {
 		result    = s.search(sLimit, p.randomProbability());
 		minLimit  = (minLimit - sLimit.conflicts); // number of actual conflicts
 		if (gLimit.conflicts != UINT64_MAX) { gLimit.conflicts -= minLimit; }
-		if (result == value_true) {
-			if (!backtrackFromModel(s)) {
-				break; // No more models requested
-			}
-			else {
-				result   = value_free; // continue enumeration
-				t        = Enumerator::progress_model;
-				if (p.restart.resetOnModel) {
-					rs.reset();
-				}
-				// After the first solution was found, we allow further restarts only if this
-				// is compatible with the enumerator used. 
-				rsLimit  = !p.restart.bounded && s.backtrackLevel() > s.rootLevel()
-					? static_cast<uint64>(-1)
-					: rs.current();
-				dsLimit  = ds.current();
+		if (result == value_true && backtrackFromModel(s)) {
+			result   = value_free; // continue enumeration
+			t        = Enumerator::progress_model;
+			if (p.restart.resetOnModel) {
+				rs.reset();
 			}
+			// After the first solution was found, we allow further restarts only if this
+			// is compatible with the enumerator used. 
+			rsLimit  = !p.restart.bounded && s.backtrackLevel() > s.rootLevel()
+				? static_cast<uint64>(-1)
+				: rs.current();
+			dsLimit  = ds.current();
 		}
 		else if (result == value_free){  // limit reached
 			rsLimit -= (!p.restart.local) * minLimit;
