@@ -454,7 +454,7 @@ void Solver::removeWatch(const Literal& p, ClauseHead* h) {
 	pList.erase_left(std::find_if(pList.left_begin(), pList.left_end(), ClauseWatch::EqHead(h)));
 }
 
-void Solver::removeUndoWatch(uint32 dl, Constraint* c) {
+bool Solver::removeUndoWatch(uint32 dl, Constraint* c) {
 	assert(dl != 0 && dl <= decisionLevel() );
 	if (levels_[dl-1].undo) {
 		ConstraintDB& uList = *levels_[dl-1].undo;
@@ -462,8 +462,10 @@ void Solver::removeUndoWatch(uint32 dl, Constraint* c) {
 		if (it != uList.end()) {
 			*it = uList.back();
 			uList.pop_back();
+			return true;
 		}
 	}
+	return false;
 }
 
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -701,7 +703,6 @@ bool Solver::test(Literal p, PostPropagator* c) {
 	freezeLevel(decisionLevel()); // can't split-off this level
 	if (propagateUntil(c)) {
 		assert(decision(decisionLevel()) == p);
-		unfreezeLevel(decisionLevel());
 		if (c) c->undoLevel(*this);
 		undoUntil(decisionLevel()-1);
 		return true;
@@ -757,15 +758,16 @@ bool Solver::forceImplied() {
 
 void Solver::undoUntil(uint32 level) {
 	assert(btLevel_ >= rootLevel_);
-	level = std::max( level, btLevel_ );
+	level      = std::max( level, btLevel_ );
 	if (level >= decisionLevel()) return;
+	uint32 num = decisionLevel() - level;
+	bool sp    = strategy_.saveProgress > 0 && ((uint32)strategy_.saveProgress) <= num;
+	bool ok    = conflict_.empty() && levels_.back().freeze == 0;
 	conflict_.clear();
 	strategy_.heuristic->undoUntil( *this, levels_[level].trailPos);
-	bool sp = strategy_.saveProgress > 0 && (decisionLevel() - level) > (uint32)strategy_.saveProgress;
-	undoLevel(false);
-	while (decisionLevel() != level) {
-		undoLevel(sp);
-	}
+	undoLevel(sp && ok);
+	while (--num) { undoLevel(sp); }
+	assert(level == decisionLevel());
 }
 
 uint32 Solver::undoUntil(uint32 level, bool popBt) {
