@@ -52,6 +52,7 @@ class ClauseCreatorTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testIntegrateEmpty);
 	CPPUNIT_TEST(testIntegrateUnit);
 	CPPUNIT_TEST(testIntegrateUnitSAT);
+	CPPUNIT_TEST(testIntegrateConflict);
 	CPPUNIT_TEST(testIntegrateAssertingConflict);
 	CPPUNIT_TEST(testIntegrateAssertingConflictBelowRoot);
 	CPPUNIT_TEST(testIntegrateConflictBelowRoot);
@@ -319,6 +320,22 @@ public:
 		CPPUNIT_ASSERT(solver().decisionLevel() == 0);
 	}
 
+	void testIntegrateConflict() {
+		solver().assume(a) && solver().propagate();
+		solver().assume(b) && solver().propagate();
+		solver().assume(c) && solver().propagate();
+		solver().force(d, 0) && solver().propagate();
+		
+		// ~a ~b ~c ~d -> conflicting@3
+		LitVec cl;
+		cl.push_back(~a); cl.push_back(~c); cl.push_back(~b); cl.push_back(~d);
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
+		CPPUNIT_ASSERT(!r.ok);
+		CPPUNIT_ASSERT(r.local != 0);
+		CPPUNIT_ASSERT(solver().hasConflict());
+	}
+
 	void testIntegrateAssertingConflict() {
 		solver().assume(a) && solver().propagate();
 		solver().assume(b) && solver().propagate();
