@@ -59,6 +59,8 @@ class ClauseTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testStrengthen);
 	CPPUNIT_TEST(testStrengthenToUnary);
 	CPPUNIT_TEST(testStrengthenContracted);
+	CPPUNIT_TEST(testStrengthenBug);
+	CPPUNIT_TEST(testStrengthenContractedNoExtend);
 	CPPUNIT_TEST(testStrengthenLocked);
 	CPPUNIT_TEST(testStrengthenLockedEarly);
 	CPPUNIT_TEST(testSimplifyTagged);
@@ -413,6 +415,46 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, c->strengthen(*solver, posLit(11)).second);
 	}
 
+	void testStrengthenBug() {
+		solver->strategies().setCompressionStrategy(6);
+		ctx.endInit(1);
+		LitVec clause;
+		clause.push_back(a1);
+		for (uint32 i = 2; i <= 6; ++i) {
+			solver->assume(negLit(8-i)) && solver->propagate();
+			clause.push_back(posLit(i));
+		}
+		ClauseHead* c = Clause::newContractedClause(*solver, clause, ClauseInfo(), 5, true);
+		solver->add(c);
+		uint32 si = c->size();
+		CPPUNIT_ASSERT(si == 5);
+		c->strengthen(*solver, posLit(4));
+		LitVec clause2;
+		c->toLits(clause2);
+		CPPUNIT_ASSERT(clause2.size() == 5);
+		for (uint32 i = 0; i != clause.size(); ++i) {
+			CPPUNIT_ASSERT(std::find(clause2.begin(), clause2.end(), clause[i]) != clause2.end() || clause[i] == posLit(4));
+		}
+	}
+
+	void testStrengthenContractedNoExtend() {
+		solver->strategies().setCompressionStrategy(4);
+		ctx.endInit(1);
+		LitVec clause;
+		clause.push_back(a1);
+		for (uint32 i = 2; i <= 6; ++i) {
+			solver->assume(negLit(8-i)) && solver->propagate();
+			clause.push_back(posLit(i));
+		}
+		ClauseHead* c = Clause::newContractedClause(*solver, clause, ClauseInfo(), 4, false);
+		solver->add(c);
+		CPPUNIT_ASSERT(c->size() == 4);
+		c->strengthen(*solver, posLit(2));
+		CPPUNIT_ASSERT(c->size() == 4);
+		solver->undoUntil(0);
+		CPPUNIT_ASSERT(c->size() == 4);
+	}
+
 	void testStrengthenLocked() {
 		Var a = ctx.addVar( Var_t::atom_var );
 		Var b = ctx.addVar( Var_t::atom_var );
