@@ -35,6 +35,7 @@ class MinimizeTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testMultiLevelWeightsAreReused);
 	CPPUNIT_TEST(testMergeComplementaryLits);
 	CPPUNIT_TEST(testMergeComplementaryLits2);
+
 	CPPUNIT_TEST(testOrder);
 	CPPUNIT_TEST(testSkipLevel);
 	CPPUNIT_TEST(testReassertAfterBacktrack);
@@ -62,6 +63,8 @@ class MinimizeTest : public CppUnit::TestFixture {
 
 	CPPUNIT_TEST(testHierarchicalSetModel);
 	CPPUNIT_TEST(testHierarchical);
+	CPPUNIT_TEST(testInconsistent);
+
 	CPPUNIT_TEST_SUITE_END(); 
 public:
 	MinimizeTest() {
@@ -77,19 +80,15 @@ public:
 	}
 	void setUp()    { newMin = 0; data = 0; }
 	void tearDown() { 
-		if (newMin) { 
-			newMin->destroy(ctx.master(), true); 
-		} 
-		if (data) {
-			data->destroy();
-		}
+		if (newMin) { newMin->destroy(ctx.master(), true);  } 
+		if (data)   { data->destroy(); }
 	}
 
 	void testEmpty() {
 		newMin = MinimizeBuilder().buildAndAttach(ctx);
 		CPPUNIT_ASSERT(countMinLits() == 0);
 	}
-
+	
 	void testOneLevelLits() {
 		WeightLitVec min;
 		ctx.addUnary(c);
@@ -115,7 +114,7 @@ public:
 		newMin = 0;
 		CPPUNIT_ASSERT(!ctx.master()->hasWatch(a, newMin));
 	}
-
+	
 	void testMultiLevelLits() {
 		MinimizeBuilder builder;
 		WeightLitVec min;
@@ -231,8 +230,8 @@ public:
 		solver.assume(~a);
 		CPPUNIT_ASSERT_EQUAL(true, solver.propagate());
 		CPPUNIT_ASSERT(newMin->sum(0) == 0 && newMin->sum(1) == 1);
-		solver.undoUntil(newMin->setModel(solver));
-		CPPUNIT_ASSERT(newMin->propagateNewOptimum(solver));
+		solver.undoUntil(newMin->commitCurrent(solver));
+		CPPUNIT_ASSERT(newMin->integrateNext(solver));
 		CPPUNIT_ASSERT(solver.isFalse(a));
 		CPPUNIT_ASSERT(solver.isFalse(b));
 		CPPUNIT_ASSERT(solver.decisionLevel() == 0);
@@ -257,10 +256,10 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(~d) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(c) && solver.propagate());
 		solver.force(~e, 0); solver.propagate();
-		CPPUNIT_ASSERT_EQUAL(uint32(3), newMin->setModel(solver));
+		CPPUNIT_ASSERT_EQUAL(uint32(3), newMin->commitCurrent(solver));
 		solver.backtrack();
 		solver.force(e, 0);
-		CPPUNIT_ASSERT_EQUAL(true, newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT_EQUAL(true, newMin->integrateNext(solver));
 		CPPUNIT_ASSERT(solver.decisionLevel() == 2 && solver.backtrackLevel() == 2);
 		CPPUNIT_ASSERT(solver.isFalse(c) && solver.isFalse(e));
 		solver.backtrack();
@@ -277,10 +276,10 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(a) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(~x) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(y) && solver.propagate());
-		CPPUNIT_ASSERT_EQUAL(uint32(2), newMin->setModel(solver));
+		CPPUNIT_ASSERT_EQUAL(uint32(2), newMin->commitCurrent(solver));
 		solver.undoUntil(1);
 		solver.setBacktrackLevel(1);
-		CPPUNIT_ASSERT_EQUAL(true, newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT_EQUAL(true, newMin->integrateNext(solver));
 		CPPUNIT_ASSERT(solver.decisionLevel() == 1 && solver.isTrue(~x));
 		solver.backtrack();
 		CPPUNIT_ASSERT(solver.decisionLevel() == 0 && solver.isTrue(~x));
@@ -306,9 +305,9 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(~d) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(c) && solver.propagate());
 		solver.force(~e, 0); solver.propagate();
-		CPPUNIT_ASSERT_EQUAL(uint32(3), newMin->setModel(solver));
+		CPPUNIT_ASSERT_EQUAL(uint32(3), newMin->commitCurrent(solver));
 		solver.undoUntil(0);
-		CPPUNIT_ASSERT_EQUAL(true, newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT_EQUAL(true, newMin->integrateNext(solver));
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(a) && solver.propagate());
 		solver.force(c, 0);
 		solver.force(d, 0);
@@ -333,9 +332,9 @@ public:
 		Solver& solver = *ctx.master();
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(a) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(c) && solver.propagate());
-		CPPUNIT_ASSERT_EQUAL(1u, newMin->setModel(solver));
+		CPPUNIT_ASSERT_EQUAL(1u, newMin->commitCurrent(solver));
 		solver.undoUntil(0);
-		CPPUNIT_ASSERT_EQUAL(true, newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT_EQUAL(true, newMin->integrateNext(solver));
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(d) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.isTrue(~a));
 		CPPUNIT_ASSERT_EQUAL(true, solver.isTrue(~b));
@@ -351,19 +350,18 @@ public:
 		newMin = MinimizeBuilder()
 			.addRule(aMin)
 			.addRule(bMin)
+			.setOptimum(0, 1)
+			.setOptimum(1, 1)
 			.buildAndAttach(ctx, MinimizeMode_t::enumerate);
 		
-		MinimizeConstraint::SumVec vec;
 		Solver& solver = *ctx.master();
-		vec.push_back(1);
-		vec.push_back(1);
-		CPPUNIT_ASSERT_EQUAL(true, setOptimum(solver, vec, false));
+		
 		CPPUNIT_ASSERT_EQUAL(true, solver.propagate());
 		
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(a) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(c) && solver.propagate());
 
-		CPPUNIT_ASSERT_EQUAL(1u, newMin->setModel(solver));
+		CPPUNIT_ASSERT_EQUAL(1u, newMin->commitCurrent(solver));
 		solver.undoUntil(0);
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(d) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(b) && solver.propagate());
@@ -385,9 +383,9 @@ public:
 		solver.force(b,0)&& solver.propagate();
 		solver.assume(c) && solver.propagate();
 		solver.force(~d,0) && solver.propagate();
-		CPPUNIT_ASSERT_EQUAL(1u, newMin->setModel(solver));
+		CPPUNIT_ASSERT_EQUAL(1u, newMin->commitCurrent(solver));
 		solver.undoUntil(1u);
-		CPPUNIT_ASSERT(newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT(newMin->integrateNext(solver));
 		CPPUNIT_ASSERT(solver.isFalse(d));
 		LitVec r1, r2;
 		solver.reason(~d, r1);
@@ -408,7 +406,7 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(a) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(b) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(c) && solver.propagate());
-		CPPUNIT_ASSERT_EQUAL(0u, newMin->setModel(solver));
+		CPPUNIT_ASSERT_EQUAL(0u, newMin->commitCurrent(solver));
 	}
 
 	void testPriorityBug() {
@@ -430,9 +428,9 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(a) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(e) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(f) && solver.propagate());
-		CPPUNIT_ASSERT_EQUAL(2u, newMin->setModel(solver));
+		CPPUNIT_ASSERT_EQUAL(2u, newMin->commitCurrent(solver));
 		solver.backtrack();
-		CPPUNIT_ASSERT_EQUAL(true, newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT_EQUAL(true, newMin->integrateNext(solver));
 		CPPUNIT_ASSERT(solver.decisionLevel() == 2);
 		solver.backtrack();
 		CPPUNIT_ASSERT_EQUAL(true, solver.propagate());
@@ -449,9 +447,9 @@ public:
 		aMin.push_back( WeightLiteral(a, 1) );
 		aMin.push_back( WeightLiteral(b, 2) );
 		aMin.push_back( WeightLiteral(c, 1) );
-		newMin = MinimizeBuilder().addRule(aMin).buildAndAttach(ctx);
+		newMin = MinimizeBuilder().addRule(aMin).setOptimum(0, 2).buildAndAttach(ctx);
 		Solver& solver = *ctx.master();
-		CPPUNIT_ASSERT(setOptimum(solver, MinimizeConstraint::SumVec(1, 2), false));
+	
 		solver.assume(a) && solver.propagate();
 		solver.setBacktrackLevel(1);
 		ctx.setProject(a.var(), true);
@@ -463,9 +461,9 @@ public:
 		solver.assume(y) && solver.propagate();
 		solver.assume(c) && solver.propagate();
 		solver.assume(e) && solver.propagate();
-		CPPUNIT_ASSERT(newMin->setModel(solver) == 3);
+		CPPUNIT_ASSERT(newMin->commitCurrent(solver) == 3);
 		solver.undoUntil(3);
-		CPPUNIT_ASSERT(newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT(newMin->integrateNext(solver));
 		CPPUNIT_ASSERT(solver.decisionLevel() == 1 && solver.isTrue(~b));
 		r.clear();
 		solver.reason(~b, r);
@@ -552,10 +550,7 @@ public:
 		ctx.setProject(b.var(), true);
 		solver.setBacktrackLevel(3);
 		opt[0] = 2;
-		CPPUNIT_ASSERT(!setOptimum(solver, opt, true));
-		CPPUNIT_ASSERT(solver.hasConflict());
-		CPPUNIT_ASSERT(solver.resolveConflict() && solver.propagate());
-		CPPUNIT_ASSERT(newMin->integrateNext(solver) == true);
+		CPPUNIT_ASSERT(setOptimum(solver, opt, true));
 		CPPUNIT_ASSERT(solver.decisionLevel() == 1 && solver.queueSize() == 3);
 		solver.propagate();
 		opt[0] = 0;
@@ -584,9 +579,9 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(a) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(b) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(d) && solver.propagate());
-		CPPUNIT_ASSERT_EQUAL(2u, newMin->setModel(solver));
+		CPPUNIT_ASSERT_EQUAL(2u, newMin->commitCurrent(solver));
 		solver.backtrack();
-		CPPUNIT_ASSERT_EQUAL(true, newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT_EQUAL(true, newMin->integrateNext(solver));
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(e) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.isTrue(~f));
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(c) && solver.propagate());
@@ -608,8 +603,8 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(b) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(~c) && solver.propagate());
 		
-		solver.undoUntil(newMin->setModel(solver));
-		CPPUNIT_ASSERT_EQUAL(true, newMin->propagateNewOptimum(solver));
+		solver.undoUntil(newMin->commitCurrent(solver));
+		CPPUNIT_ASSERT_EQUAL(true, newMin->integrateNext(solver));
 		
 		CPPUNIT_ASSERT_EQUAL(true, solver.isTrue(~b));
 		CPPUNIT_ASSERT_EQUAL(true, solver.isTrue(~c));
@@ -627,9 +622,9 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(~a) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.force(b, 0) && solver.propagate());
 			
-		CPPUNIT_ASSERT(newMin->setModel(solver) == 0);
+		CPPUNIT_ASSERT(newMin->commitCurrent(solver) == 0);
 		solver.backtrack();
-		CPPUNIT_ASSERT_EQUAL(false, newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT_EQUAL(false, newMin->integrateNext(solver));
 	}
 
 	void testMultiAssignment() {
@@ -684,18 +679,18 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver.force(e,0) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.force(~f,0) && solver.propagate());
 		
-		solver.undoUntil(newMin->setModel(solver));
+		solver.undoUntil(newMin->commitCurrent(solver));
 		CPPUNIT_ASSERT(3 == solver.decisionLevel());
 		solver.setBacktrackLevel(3);
-		CPPUNIT_ASSERT(newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT(newMin->integrateNext(solver));
 		CPPUNIT_ASSERT_EQUAL(true, solver.force(f,0) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.isTrue(~d));
 		CPPUNIT_ASSERT_EQUAL(true, solver.isTrue(~e));
 		
-		uint32 dl = newMin->setModel(solver);
+		uint32 dl = newMin->commitCurrent(solver);
 		CPPUNIT_ASSERT(2 == dl);
 		solver.backtrack();
-		CPPUNIT_ASSERT(newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT(newMin->integrateNext(solver));
 		CPPUNIT_ASSERT_EQUAL(true, solver.isTrue(~x));
 		CPPUNIT_ASSERT_EQUAL(true, solver.isTrue(~f));
 		CPPUNIT_ASSERT_EQUAL(true, solver.isTrue(~d));
@@ -722,14 +717,14 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(b) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(c) && solver.propagate());
 
-		solver.undoUntil(newMin->setModel(solver));
-		CPPUNIT_ASSERT(newMin->propagateNewOptimum(solver) && 1 == solver.decisionLevel());
+		solver.undoUntil(newMin->commitCurrent(solver));
+		CPPUNIT_ASSERT(newMin->integrateNext(solver) && 1 == solver.decisionLevel());
 		CPPUNIT_ASSERT(solver.propagate());
 		
-		uint32 dl = newMin->setModel(solver);
+		uint32 dl = newMin->commitCurrent(solver);
 		CPPUNIT_ASSERT(dl == 0);
 		solver.undoUntil(dl);
-		CPPUNIT_ASSERT(newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT(newMin->integrateNext(solver));
 		CPPUNIT_ASSERT(solver.isTrue(~a));
 		CPPUNIT_ASSERT(solver.value(b.var()) == value_free);
 	}
@@ -742,8 +737,8 @@ public:
 		Solver& solver = *ctx.master();
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(a) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(b) && solver.force(~c,0) && solver.propagate());
-		solver.undoUntil(newMin->setModel(solver));
-		newMin->propagateNewOptimum(solver);
+		solver.undoUntil(newMin->commitCurrent(solver));
+		newMin->integrateNext(solver);
 		CPPUNIT_ASSERT(0u == solver.decisionLevel() && solver.isFalse(a));
 	}
 
@@ -755,7 +750,7 @@ public:
 		Solver& solver = *ctx.master();
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(a) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(b) && solver.propagate());
-		uint32 DL = newMin->setModel(solver);
+		uint32 DL = newMin->commitCurrent(solver);
 		CPPUNIT_ASSERT(DL < solver.decisionLevel());
 		solver.undoUntil(DL);
 		CPPUNIT_ASSERT(0 == newMin->shared()->optimum()->opt[0]);
@@ -763,7 +758,7 @@ public:
 		solver.clearAssumptions();
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(~a) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(~b) && solver.propagate());
-		DL = newMin->setModel(solver);
+		DL = newMin->commitCurrent(solver);
 		CPPUNIT_ASSERT(DL == UINT32_MAX);
 		CPPUNIT_ASSERT(-2 == newMin->shared()->optimum()->opt[0]);
 	}
@@ -782,8 +777,8 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(b) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(c) && solver.propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver.assume(d) && solver.propagate());
-		solver.undoUntil(newMin->setModel(solver));
-		newMin->propagateNewOptimum(solver);
+		solver.undoUntil(newMin->commitCurrent(solver));
+		newMin->integrateNext(solver);
 		CPPUNIT_ASSERT(2 == solver.decisionLevel());
 	}
 
@@ -810,7 +805,7 @@ public:
 		CPPUNIT_ASSERT(r.size() == 1 && r[0] == minAssume);
 
 		solver.clearAssumptions();
-		newMin->propagateNewOptimum(solver);
+		newMin->integrateNext(solver);
 		CPPUNIT_ASSERT(solver.numAssignedVars() == 0);
 	}
 
@@ -836,9 +831,9 @@ public:
 		solver.assume(e); solver.propagate();
 		solver.assume(f); solver.propagate();
 		data->setMode(MinimizeMode_t::optimize, true);
-		CPPUNIT_ASSERT(newMin->setModel(solver) == 2);
+		CPPUNIT_ASSERT(newMin->commitCurrent(solver) == 2);
 		solver.undoUntil(2);
-		newMin->propagateNewOptimum(solver);
+		newMin->integrateNext(solver);
 		CPPUNIT_ASSERT(solver.isFalse(c));
 		CPPUNIT_ASSERT(solver.numAssignedVars() == 3);
 	}
@@ -863,14 +858,27 @@ public:
 		ctx.endInit();
 		solver.assume(a);
 		solver.propagate();
-		uint32 x = newMin->setModel(solver);
+		uint32 x = newMin->commitCurrent(solver);
 		CPPUNIT_ASSERT(x == 0);
 		solver.backtrack();
-		CPPUNIT_ASSERT(newMin->propagateNewOptimum(solver));
+		CPPUNIT_ASSERT(newMin->integrateNext(solver));
 		CPPUNIT_ASSERT(solver.propagate() == true);
-		x = newMin->setModel(solver);
+		x = newMin->commitCurrent(solver);
 		CPPUNIT_ASSERT(x == UINT32_MAX);
 	}
+	void testInconsistent() {
+		MinimizeBuilder builder;
+		WeightLitVec min;
+		min.push_back( WeightLiteral(a, 1) );
+		min.push_back( WeightLiteral(b, 1) );
+		min.push_back( WeightLiteral(c, 1) );
+		builder.addRule(min);
+		builder.setOptimum(0, 1);
+		ctx.addUnary(a);
+		ctx.addUnary(b);
+		CPPUNIT_ASSERT(builder.build(ctx) == 0);
+		CPPUNIT_ASSERT(builder.buildAndAttach(ctx) == 0);
+	}
 private:
 	uint32 countMinLits() const {
 		uint32 lits = 0;
@@ -878,16 +886,14 @@ private:
 		for (; !isSentinel(it->first); ++it, ++lits) { ; }
 		return lits;
 	}
-	bool setOptimum(Solver& s, const SharedMinimizeData::SumVec& vec, bool less) {
+	
+	bool setOptimum(Solver& s, SharedMinimizeData::SumVec& vec, bool less) {
 		SharedMinimizeData* data = const_cast<SharedMinimizeData*>(newMin->shared());
+		if (!less) {
+			vec.back() += 1;
+		}
 		data->setOptimum(&vec[0]);
-		if (less) {
-			return newMin->integrateNext(s);	
-		}
-		else {
-			newMin->restoreOptimum();
-			return newMin->propagateNewOptimum(s);
-		}
+		return newMin->integrateNext(s);
 	}
 	MinimizeConstraint* createMin(Solver& s, SharedMinimizeData* data) {
 		return MinimizeBuilder::attach(s, data);
