@@ -59,7 +59,7 @@ struct TestingConstraint : public LearntConstraint {
 	bool* deleted;
 };
 struct TestingPostProp : public PostPropagator {
-	explicit TestingPostProp(bool cfl) : props(0), resets(0), symMods(0), prio(PostPropagator::priority_single), conflict(cfl), symModel(false) {}
+	explicit TestingPostProp(bool cfl) : props(0), resets(0), prio(PostPropagator::priority_single), conflict(cfl) {}
 	bool propagate(Solver&) {
 		++props;
 		return !conflict;
@@ -68,16 +68,10 @@ struct TestingPostProp : public PostPropagator {
 		++resets;
 	}
 	uint32 priority() const { return prio; }
-	bool nextSymModel(Solver& s, bool expand) {
-		++symMods;
-		return expand && symModel; 
-	}
 	int props;
 	int resets;
-	int symMods;
 	uint32 prio;
 	bool conflict;
-	bool symModel; 
 };
 
 class SolverTest : public CppUnit::TestFixture {
@@ -118,7 +112,6 @@ class SolverTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testPropagateCallsPostProp);
 	CPPUNIT_TEST(testPropagateCallsResetOnConflict);
 	CPPUNIT_TEST(testPostpropPriority);
-	CPPUNIT_TEST(testPostpropSymModel);
 
 	CPPUNIT_TEST(testSimplifyRemovesSatBinClauses);
 	CPPUNIT_TEST(testSimplifyRemovesSatTernClauses);
@@ -706,40 +699,6 @@ public:
 		CPPUNIT_ASSERT(p1->next == p3 && p3->next == p2);
 	}
 
-	void testPostpropSymModel() {
-		Literal a = posLit(ctx.addVar(Var_t::atom_var));
-		Literal b = posLit(ctx.addVar(Var_t::atom_var));
-		Solver& s = ctx.startAddConstraints();
-		TestingPostProp* p1 = new TestingPostProp(false);
-		TestingPostProp* p2 = new TestingPostProp(false);
-		s.addPost(p1);
-		s.addPost(p2);
-		ctx.endInit();
-		p1->symModel = false;
-		p2->symModel = true;
-		ValueRep v = s.search(1, 1);
-		CPPUNIT_ASSERT_EQUAL(true, s.nextSymModel(true));
-		CPPUNIT_ASSERT_EQUAL(1, p1->symMods);
-		CPPUNIT_ASSERT_EQUAL(1, p2->symMods);
-		p2->symModel = false;
-		CPPUNIT_ASSERT_EQUAL(false, s.nextSymModel(true));
-		CPPUNIT_ASSERT_EQUAL(1, p1->symMods);
-		CPPUNIT_ASSERT_EQUAL(2, p2->symMods);
-		s.undoUntil(0);
-		v = s.search(1, 1);
-		p1->symModel = true;
-		CPPUNIT_ASSERT_EQUAL(true, s.nextSymModel(true));
-		CPPUNIT_ASSERT_EQUAL(2, p1->symMods);
-		CPPUNIT_ASSERT_EQUAL(2, p2->symMods);
-		CPPUNIT_ASSERT_EQUAL(true, s.nextSymModel(true));
-		CPPUNIT_ASSERT_EQUAL(3, p1->symMods);
-		CPPUNIT_ASSERT_EQUAL(2, p2->symMods);
-		p1->symModel = false;
-		CPPUNIT_ASSERT_EQUAL(false, s.nextSymModel(true));
-		CPPUNIT_ASSERT_EQUAL(4, p1->symMods);
-		CPPUNIT_ASSERT_EQUAL(3, p2->symMods);
-	}
-
 	void testSimplifyRemovesSatBinClauses() {
 		Literal a = posLit(ctx.addVar(Var_t::atom_var));
 		Literal b = posLit(ctx.addVar(Var_t::atom_var));
@@ -1032,15 +991,16 @@ public:
 		Var a = ctx.addVar( Var_t::atom_var );
 		Var b = ctx.addVar( Var_t::atom_var );
 		Var c = ctx.addVar( Var_t::atom_var );
+		Var d = ctx.addVar( Var_t::atom_var );
 		Solver& s = ctx.startAddConstraints();
 		ctx.addBinary(posLit(a), negLit(b));
 		ctx.addBinary(negLit(a), posLit(b));
 		ctx.addBinary(negLit(a), negLit(b));
-		ctx.addBinary(posLit(a), posLit(b));
 		ctx.endInit();
 		s.simplify();
 		s.assume(posLit(c));
 		s.pushRootLevel();
+		s.assume(posLit(a));
 		CPPUNIT_ASSERT_EQUAL(value_free, s.search(1, -1, 0));
 		CPPUNIT_ASSERT_EQUAL(1u, s.decisionLevel());
 	} 
