@@ -55,12 +55,18 @@ StringSlice parseValue(const StringSlice& in, T& out, ...) {
 }
 
 struct FlagStr { 
-  std::string str; bool val; 
+  typedef bool (*parser_type)(const std::string&, bool&);
+	std::string str; bool val; 
   static const FlagStr* find(const std::string& s);
   static const FlagStr* find(const char* s);
 	static const FlagStr* findImpl(const char* s, std::size_t len);
 	//! parses "1" | "0" | "yes" | "no" | "true" | "false" | "on" | "off"
 	static StringSlice parse(const StringSlice& slice, bool& out, int);
+	static parser_type parser(int x) {
+		return x ? store_true : store_false;
+	}
+	static bool store_true(const std::string&, bool&);
+	static bool store_false(const std::string&, bool&);
   static FlagStr map_s[8];
 };
 
