@@ -38,13 +38,17 @@
 #include <cctype>
 using namespace std;
 
-namespace ProgramOptions { namespace detail {
-
-static void defaultFormat(std::string& temp, const ProgramOptions::Option& o, std::size_t maxW) {
+namespace ProgramOptions {
+///////////////////////////////////////////////////////////////////////////////
+// DefaultFormat
+///////////////////////////////////////////////////////////////////////////////
+std::size_t DefaultFormat::format(std::vector<char>& buf, const Option& o, std::size_t maxW) {
+	buf.clear();
 	size_t bufSize = std::max(maxW, o.maxColumn()) + 3;
-	char*  buffer  = new char[bufSize];
+	buf.resize(bufSize);
+	char*  buffer  = &buf[0];
 	size_t n       = sprintf(buffer, "  --%s", o.name().c_str());
-	const char* arg= o.argDescription();
+	const char* arg= o.argName();
 	if (o.value()->isImplicit() && *arg) {
 		n += sprintf(buffer+n, "[=%s]", arg);
 	}
@@ -54,94 +58,145 @@ static void defaultFormat(std::string& temp, const ProgramOptions::Option& o, st
 	if (!o.value()->isImplicit()) {
 		n += sprintf(buffer+n, "%c%s", (!o.alias()?'=':' '), arg);
 	}
-	if (*o.description()) {
-		if (n < maxW) n += sprintf(buffer+n, "%-*.*s", int(maxW-n), int(maxW-n), " ");
-		buffer[n++] = ':';
-		buffer[n++] = ' ';
-		assert(n <= bufSize);
-		temp.append(buffer, n);
-		o.formatDescription(temp);
+	if (n < maxW) n += sprintf(buffer+n, "%-*.*s", int(maxW-n), int(maxW-n), " ");
+	assert(n <= bufSize);
+	return n;
+}
+std::size_t DefaultFormat::format(std::vector<char>& buf, const char* desc, const Value& val, std::size_t) {
+	std::size_t minS = strlen(desc);
+	const char* temp = 0;
+	if (!desc)  desc = "";
+	buf.clear();
+	buf.reserve(minS+2);
+	buf.push_back(':');
+	buf.push_back(' ');
+	for (const char* look;; ++desc) {
+		look = desc;
+		while (*look && *look != '%') {
+			++look;
+		}
+		if (look != desc)      { buf.insert(buf.end(), desc, look); }
+		if      (!*look++ || !*look) break;
+		else if (*look == 'D') { temp = val.defaultsTo(); }
+		else if (*look == 'A') { temp = val.arg(); }
+		else if (*look == 'I') { temp = val.implicit(); }
+		else                   { buf.push_back(*look); }
+		if (temp)              { buf.insert(buf.end(), temp, temp + strlen(temp)); }
+		desc = look;
+		temp = 0;
 	}
-	delete [] buffer;
+	buf.push_back('\n');
+	return buf.size();
+}
+std::size_t DefaultFormat::format(std::vector<char>& buffer, const OptionGroup& grp) {
+	buffer.clear();
+	if (grp.caption().length()) {
+		buffer.reserve(grp.caption().length() + 4);
+		buffer.push_back('\n');
+		buffer.insert(buffer.end(), grp.caption().begin(), grp.caption().end());
+		buffer.push_back(':');
+		buffer.push_back('\n');
+		buffer.push_back('\n');
+	}
+	return buffer.size();
+}
+void OstreamWriter::write(const std::vector<char>& buf, std::size_t n) {
+	if (n) { out.write(&buf[0], n); }
+}
+void StringWriter::write(const std::vector<char>& buf, std::size_t n) {
+	if (n) { out.append(&buf[0], n); }
+}
+void FileWriter::write(const std::vector<char>& buf, std::size_t n) {
+	if (n) { fwrite(&buf[0], 1, n, out); }
 }
-} // namespace detail
 ///////////////////////////////////////////////////////////////////////////////
 // class Value
 ///////////////////////////////////////////////////////////////////////////////
-std::string Value::empty_string_s("");
+Value::Value(byte_t flagSet, State initial)
+	: state_(initial)
+	, flags_(flagSet)
+	, descFlag_(0)
+	, optAlias_(0) {
+	desc_.value = 0;
+}
 
-Value::Value(byte_t flags, ValueState initial) : flagSet_(flags), state_(initial)  {
-	extra_[default_value] = extra_[implicit_value] = 0;
-}
 Value::~Value() {
-	release(default_value);
-	release(implicit_value);
+	if (descFlag_ == desc_pack) {
+		::operator delete(desc_.pack);
+	}
 }
-void Value::release(ExtraValue e) {
-	byte_t f = extra2Flag(e);
-	if (hasFlag(f)) {
-		delete extra_[e];
-		clearFlag(f);
-	}
-	extra_[e] = 0;
+
+const char* Value::arg() const {
+	const char* x = desc(desc_name);
+	if (x) return x;
+	return isFlag() ? "" : "<arg>";
 }
-void Value::setExtraValue(ExtraValue e, const char* str) {
-	if (!str) return;
-	release(e);
-	ExtraValue other = static_cast<ExtraValue>(1 - e);
-	if (!*str) {
-		// share empty
-		extra_[e] = &empty_string_s;
+
+Value* Value::desc(DescType t, const char* n) {
+	if (n == 0) return this;
+	if (t == desc_implicit) { 
+		setProperty(property_implicit);
+		if (!*n) return this;
 	}
-	else if (const FlagStr* x = FlagStr::find(str)) {
-		// share with flag
-		extra_[e] = &x->str;
+	if (descFlag_ == 0 || descFlag_ == t) {
+		desc_.value = n;
+		descFlag_   = t;
+		return this;
 	}
-	else if (extra_[other] && *extra_[other] == str) {
-		// share with other if possible
-		extra_[e] = extra_[other];
+	if (descFlag_ != desc_pack) {
+		const char* oldVal = desc_.value;
+		unsigned    oldKey = descFlag_ >> 1u;
+		desc_.pack   = (const char**)::operator new(sizeof(const char*[3]));
+		desc_.pack[0]= desc_.pack[1] = desc_.pack[2] = 0;
+		descFlag_    = desc_pack;
+		desc_.pack[oldKey] = oldVal;
 	}
-	else {
-		// create new owned string
-		extra_[e] = new std::string(str);
-		setFlag( extra2Flag(e) );
+	desc_.pack[t>>1u] = n;
+	return this;
+}
+const char* Value::desc(DescType t) const {
+	if (descFlag_ == t || descFlag_ == desc_pack) {
+		return descFlag_ == t
+			? desc_.value
+			: desc_.pack[t >> 1u];
 	}
+	return 0;
+}
+
+const char* Value::implicit() const {
+	if (!hasProperty(property_implicit)) return 0;
+	const char* x = desc(desc_implicit);
+	return x ? x : "";
+}
+
+bool Value::parse(const std::string& name, const std::string& value, State st) {
+	if (!value.empty() || !isImplicit()) return state(doParse(name, value), st);
+	const char*    x = implicit();
+	assert(x);
+	const FlagStr* y = FlagStr::find(x);
+	return state(doParse(name, y ? y->str:x), st);
 }
 ///////////////////////////////////////////////////////////////////////////////
 // class Option
 ///////////////////////////////////////////////////////////////////////////////
-Option::Option(const string& longName, char alias, const char* desc, Value* v, const char* arg)
+Option::Option(const string& longName, char alias, const char* desc, Value* v)
 	: name_(longName)
+	, description_(desc ? desc : "")
 	, value_(v) {
 	assert(v);
 	assert(!longName.empty());
-	if (arg == 0)  arg  = "";
-	if (desc == 0) desc = "";
-	// [alias|arg|0|desc|0]
-	std::size_t nDesc   = strlen(desc);
-	std::size_t nArg    = strlen(arg);
-	std::size_t bufSize = (nDesc + nArg + 3);
-	char*       buffer  = new char[bufSize];
-	char*       b       = buffer;
-	*b++                = alias;
-	std::memcpy(b, arg, nArg*sizeof(char));
-	b                  += nArg;
-	*b++                = '\0';
-	std::memcpy(b, desc, nDesc*sizeof(char));
-	buffer[bufSize-1]   = '\0';
-	description_        = buffer;
+	value_->alias(alias);
 }
 
 Option::~Option() {
 	delete value_;
-	delete [] description_;
 }
 std::size_t Option::maxColumn() const {
 	std::size_t col = 4 + name_.size(); //  --name
 	if (alias()) {
 		col += 3; // ,-o
 	}
-	std::size_t argN = strlen(argDescription());
+	std::size_t argN = strlen(argName());
 	if (argN) {
 		col += (argN + 1); // =arg
 		if (value()->isImplicit()) {
@@ -151,55 +206,16 @@ std::size_t Option::maxColumn() const {
 	return col;
 }
 
-void Option::formatDescription(std::string& out) const {
-	const char* desc = description();
-	std::size_t minS = strlen(desc);
-	std::string argD(argDescription() ? argDescription() : "");
-	out.reserve(out.size() + minS);
-	for (;; ++desc) {
-		while (*desc && *desc != '%') {
-			out += *desc++;
-		}
-		if (!*desc++ || !*desc) break;
-		if (*desc == 'D') {
-			out += value()->defaultValue();
-		}
-		else if (*desc == 'A') {
-			out += argD;
-		}
-		else {
-			out += *desc;
-		}
-	}
-}
-
-const char* Option::description() const {
-	return &description_[2] + strlen(arg());
-}
-const char* Option::arg() const {
-	return &description_[1];
-}
-const char* Option::argDescription() const {
-	if (*arg() || value_->isImplicit()) {
-		return arg();
-	}
-	return "<arg>";
-}
 bool Option::assignDefault() const {
-	if (value()->hasDefault() && value()->state() != Value::value_defaulted) {
-		return value()->parse(name(), value()->defaultValue(), true);
+	if (value()->defaultsTo() != 0 && value()->state() != Value::value_defaulted) {
+		return value()->parse(name(), value()->defaultsTo(), Value::value_defaulted);
 	}
 	return true;
 }
-const std::string& Option::defaultValue() const { return value()->defaultValue(); }
 ///////////////////////////////////////////////////////////////////////////////
 // class OptionGroup
 ///////////////////////////////////////////////////////////////////////////////
-OptionGroup::OptionGroup(const std::string& caption) : caption_(caption) {}
-OptionGroup OptionGroup::createHidden() {
-	return OptionGroup(std::string("\0", 1));
-}
-
+OptionGroup::OptionGroup(const std::string& caption) : caption_(caption), hide_(false) {}
 OptionGroup::~OptionGroup() { }
 
 OptionInitHelper OptionGroup::addOptions() {
@@ -219,31 +235,21 @@ std::size_t OptionGroup::maxColumn() const {
 	return maxW;
 }
 
-void OptionGroup::format(std::string& out, FormatFunction f, size_t maxW) const {
-	if (f == 0) f = detail::defaultFormat;
-	if (caption().length()) {
-		out += "\n";
-		out += !hidden() ? caption() : "Hidden Options";
-		out += ":\n\n";
-	}
+void OptionGroup::format(OptionOutput& out, size_t maxW) const {
 	for (option_iterator it = options_.begin(), end = options_.end(); it != end; ++it) {
-		f(out, **it, maxW);
-		out += "\n";
+		out.printOption(**it, maxW);
 	}
 }
-bool OptionGroup::hidden() const {
-	return caption_.size() == 1 && caption_[0] == '\0';
-}
 ///////////////////////////////////////////////////////////////////////////////
 // class OptionInitHelper
 ///////////////////////////////////////////////////////////////////////////////
 OptionInitHelper::OptionInitHelper(OptionGroup& owner)
 	: owner_(&owner) { }
 
-OptionInitHelper& OptionInitHelper::operator()(const char* name, Value* val, const char* desc, const char* extra) {
+OptionInitHelper& OptionInitHelper::operator()(const char* name, Value* val, const char* desc) {
 	std::auto_ptr<Value> value(val);
 	if (!name || !*name) {
-		throw BadOptionName("");
+		throw Error("Invalid empty option name");
 	}
 	const char* n = strchr(name, ',');
 	string longName; char shortName = 0;
@@ -255,11 +261,11 @@ OptionInitHelper& OptionInitHelper::operator()(const char* name, Value* val, con
 		++n;
 		if (!*n || *(n+1)) {
 			// empty or too long short name
-			throw BadOptionName(name);
+			throw Error(std::string("Invalid short name '").append(name).append("'"));
 		}
 		shortName = *n;
 	}
-	owner_->addOption(auto_ptr<Option>(new Option(longName, shortName, desc, value.release(), extra)));
+	owner_->addOption(auto_ptr<Option>(new Option(longName, shortName, desc, value.release())));
 	return *this;
 }
 ///////////////////////////////////////////////////////////////////////////////
@@ -288,6 +294,7 @@ OptionContext& OptionContext::add(const OptionGroup& options) {
 	for (option_iterator it = options.begin(), end = options.end(); it != end; ++it) {
 		insertOption(k, *it);
 	}
+	groups_[k].hidden( options.hidden() );
 	return *this;
 }
 
@@ -307,12 +314,12 @@ void OptionContext::insertOption(size_t groupId, const SharedOptPtr& opt) {
 		char sName[2] = {'-', opt->alias()};
 		std::string shortName(sName, 2);
 		if (!index_.insert(Name2Key::value_type(shortName, k)).second) {
-			throw DuplicateOption(l, caption());
+			throw DuplicateOption(caption(), l);
 		}
 	}
 	if (!l.empty()) {
 		if (!index_.insert(Name2Key::value_type(l, k)).second) {
-			throw DuplicateOption(l, caption());
+			throw DuplicateOption(caption(), l);
 		}
 	}
 	options_.push_back(opt);
@@ -322,16 +329,16 @@ void OptionContext::insertOption(size_t groupId, const SharedOptPtr& opt) {
 OptionContext::option_iterator OptionContext::find(const char* key, FindType t) const {
 	PrefixRange r = findImpl(key, t);
 	if (std::distance(r.first, r.second) == 0) {
-		throw UnknownOption(caption() + "::" + (key ? key : ""));
+		throw UnknownOption(caption(), (key ? key : ""));
 	}
 	if (std::distance(r.first, r.second) > 1) {
-		std::string str = "'";
-		str += key;
-		str += "' could be:";
+		std::string str;
 		for (; r.first != r.second; ++r.first) { 
-			str += "\n  " + r.first->first;
+			str += "  ";
+			str += r.first->first;
+			str += "\n";
 		}
-		throw AmbiguousOption(str);
+		throw AmbiguousOption(caption(), key, str);
 	}
 	return options_.begin() + r.first->second;
 }
@@ -365,22 +372,23 @@ OptionContext::PrefixRange OptionContext::findImpl(const char* key, FindType t)
 	return PrefixRange(index_.end(), index_.end()); 
 }
 
-std::string OptionContext::description(FormatFunction f, bool addHidden) const {
-	if (f == 0) f = detail::defaultFormat;
+OptionOutput& OptionContext::description(OptionOutput& out, bool addHidden) const {
 	size_t maxW = 23;
 	for (size_t i = 0; i != groups(); ++i) {
 		maxW = std::max(maxW, groups_[i].maxColumn());
 	}
-	std::string temp; temp.reserve(1024);
-	std::string out;
-	for (int g = 0; g < 2; ++g) {
-		// print all sub-groups followed by main group
-		for (std::size_t i = (g == 0), end = (g == 0) ? groups_.size() : 1; i < end; ++i) {
-			temp.clear();
-			if (!groups_[i].hidden() || addHidden) {
-				groups_[i].format(temp, f, maxW);
+	if (out.printContext(*this)) {
+		// print all visible sub-groups
+		for (std::size_t i = 1; i < groups_.size(); ++i) {
+			if (!groups_[i].hidden() && out.printGroup(groups_[i])) {
+				groups_[i].format(out, maxW);
+			}
+		}
+		// print main group and all hidden sub-groups
+		for (std::size_t i = 0, end = addHidden ? groups_.size() : 0 + !groups_.empty(); i < end; ++i) {
+			if ((i == 0 || groups_[i].hidden()) && out.printGroup(groups_[i])) {
+				groups_[i].format(out, maxW);
 			}
-			out  += temp;
 		}
 	}
 	return out;
@@ -395,12 +403,12 @@ std::string OptionContext::defaults(std::size_t n) const {
 			if (!groups_[i].hidden()) {
 				for (option_iterator it = groups_[i].begin(), end = groups_[i].end(); it != end; ++it) {
 					const Option& o = **it;
-					if (o.value()->hasDefault()) {
+					if (o.value()->defaultsTo()) {
 						x = defs.size();
 						defs += "--";
 						defs += o.name();
 						defs += '=';
-						defs += o.value()->defaultValue();
+						defs += o.value()->defaultsTo();
 						defs += ' ';
 						if ( (line += (defs.size() - x)) > 64) {
 							defs += '\n';
@@ -415,7 +423,8 @@ std::string OptionContext::defaults(std::size_t n) const {
 	return defs;	
 }
 std::ostream& operator<<(std::ostream& os, const OptionContext& grp) {
-	os << grp.description(detail::defaultFormat);
+	StreamOut out(os);
+	grp.description(out, false);
 	return os;
 }
 
@@ -423,12 +432,7 @@ bool OptionContext::assignDefaults(const ParsedOptions& opts) const {
 	for (option_iterator it = begin(), end = this->end(); it != end; ++it) {
 		const Option& o = **it;
 		if (opts.count(o.name()) == 0 && !o.assignDefault()) {
-			std::string d("'");
-			d += o.defaultValue();
-			d += "': invalid default value for Option '";
-			d += o.name();
-			d += "'";
-			throw BadValue(d);	
+			throw ValueError(caption(), ValueError::invalid_default, o.name(), o.value()->defaultsTo());
 		}
 	}
 	return true;
@@ -440,30 +444,39 @@ ParsedOptions::ParsedOptions()  {}
 ParsedOptions::~ParsedOptions() { parsed_.clear(); }
 bool ParsedOptions::assign(const ParsedValues& p) {
 	if (!p.ctx) return false;
-	p.prepare();
-	// assign parsed values
-	for (ParsedValues::iterator it = p.begin(), end = p.end(); it != end; ++it) {
-		const Option& o = *it->first;;
-		if (!assign(o, it->second)) {
-			std::string d("'");
-			d += it->second;
-			d += "': invalid value for Option '";
-			d += o.name();
-			d += "'";
-			throw BadValue(d);	
+	struct Assign {
+		Assign(ParsedOptions* x) : self(x) {}
+		void assign(const ParsedValues& p) {
+			begin = it = p.begin();
+			// assign parsed values
+			for (ParsedValues::iterator end = p.end(); it != end; ++it) {
+				const Option& o = *it->first;;
+				self->assign(o, it->second);
+			}
 		}
-	}
+		~Assign() { 
+			for (ParsedValues::iterator x = begin, end = this->it; x != end; ++x) {
+				self->parsed_.insert(x->first->name());
+			}
+		}
+		ParsedOptions* self;
+		ParsedValues::iterator begin;
+		ParsedValues::iterator it;
+	} scoped(this);
+	scoped.assign(p);
 	return true;
 }
-bool ParsedOptions::assign(const Option& o, const std::string& value) {
-	if (parsed_.count(o.name()) > 0 && !o.value()->isComposing()) {
-		return true;
+void ParsedOptions::assign(const Option& o, const std::string& value) {
+	unsigned badState = 0;
+	if (!o.value()->isComposing()) {
+		if (parsed_.count(o.name())) { return ; }
+		badState = (Value::value_parsed & o.value()->state());
 	}
-	if (!o.value()->parse(o.name(), value)) {
-		return false;
+	if (badState || !o.value()->parse(o.name(), value, Value::value_parsed)) {
+		badState 
+			? throw ValueError("", ValueError::multiple_occurences, o.name(), value)
+			: throw ValueError("", ValueError::invalid_default, o.name(), value);
 	}
-	parsed_.insert(o.name());
-	return true;
 }
 ///////////////////////////////////////////////////////////////////////////////
 // class ParsedValues
@@ -482,31 +495,6 @@ void ParsedValues::add(const std::string& name, const std::string& value) {
 		add(*it, value);
 	}
 }
-
-void ParsedValues::prepare() const {
-	if (!prepared_ && parsed_.size() > 1) {
-		LessFirst<OptionAndValue> lessKey;
-		std::sort(parsed_.begin(), parsed_.end(), lessKey);
-		for (Values::const_iterator it = parsed_.begin()
-			, next = parsed_.begin()+1
-			, end  = parsed_.end(); next != end; ) {
-			if (it->first.get() != next->first.get()) {
-				it = next++;
-			}
-			else if (!it->first->value()->isComposing()) {
-				std::string d("Option '");
-				d += it->first->name();
-				d += "'";
-				throw MultipleOccurences(d);
-			}
-			else {
-				it   = std::upper_bound(parsed_.begin(), parsed_.end(), *it, lessKey);
-				next = it != end ? it+1 : end;
-			}
-		}
-	}
-	prepared_ = true;
-}
 ///////////////////////////////////////////////////////////////////////////////
 // class OptionParser
 ///////////////////////////////////////////////////////////////////////////////
@@ -537,7 +525,9 @@ SharedOptPtr OptionParser::getOption(const char* name, OptionType t) const {
 		return *pv_.ctx->find(name, ft);
 	}
 	catch (const UnknownOption&) {
-		if (allowUnreg_) { return SharedOptPtr(0); }
+		if (allowUnreg_) {
+			return SharedOptPtr(0);
+		}
 		throw;
 	}
 }
@@ -617,7 +607,7 @@ private:
 					return true;
 				}
 				else {
-					throw runtime_error(string("required parameter missing after -") + optn);
+					throw SyntaxError(SyntaxError::missing_value, optn);
 				}
 			}
 			else {
@@ -638,10 +628,12 @@ private:
 		if ( (o = getOption(name.c_str(), long_opt)).get() ) {
 			if (!o->value()->isImplicit() && value.empty()) {
 				if (const char* v = next()) { value = v; }
-				else { throw runtime_error(string("required parameter missing after --") + optName); }
+				else { throw SyntaxError(SyntaxError::missing_value, name); }
+			}
+			else if (o->value()->isFlag() && !value.empty()) {
+				// flags don't have values
+				throw SyntaxError(SyntaxError::extra_value, name);
 			}
-			// value given with 'opt=value' are always used, even
-			// if opt is a flag
 			addOptionValue(o, value);
 			return true;
 		}
@@ -773,7 +765,7 @@ private:
 				sectionValue += line;
 			}
 			else {
-				throw std::runtime_error("illegal option file format");
+				throw SyntaxError(SyntaxError::invalid_format, line);
 			}
 		}
 		if (inSection) { // file does not end with an empty line
@@ -807,4 +799,73 @@ ParsedValues parseCfgFile(std::istream& in, OptionContext& o, bool allowUnreg) {
 	return p.parse();
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// Errors
+///////////////////////////////////////////////////////////////////////////////    
+static std::string quote(const std::string& x) {
+	return std::string("'").append(x).append("'");
+}
+static std::string format(SyntaxError::Type t, const std::string& key) {
+	std::string ret("SyntaxError: ");
+	ret += quote(key);
+	switch (t) {
+		case SyntaxError::missing_value: ret += " requires a value!";       break;
+		case SyntaxError::extra_value:   ret += " does not take a value!";  break;
+		case SyntaxError::invalid_format:ret += " unrecognized line!";      break;
+		default:                         ret += " unknown syntax!";
+	};
+	return ret;
+}
+static std::string format(ContextError::Type t, const std::string& ctx, const std::string& key, const std::string& alt) {
+	std::string ret;
+	if (!ctx.empty()) { ret += "In context "; ret += quote(ctx); ret += ": "; }
+	switch (t) {
+		case ContextError::duplicate_option: ret += "duplicate option: "; break;
+		case ContextError::unknown_option:   ret += "unknown option: ";   break;
+		case ContextError::ambiguous_option: ret += "ambiguous option: "; break;
+		default:                             ret += "unknown error in: ";
+	};
+	ret += quote(key);
+	if (t == ContextError::ambiguous_option && !alt.empty()) {
+		ret += " could be:\n";
+		ret += alt;
+	}
+	return ret;
+}
+static std::string format(ValueError::Type t, const std::string& ctx, const std::string& key, const std::string& value) {
+	std::string ret; const char* x = "";
+	if (!ctx.empty()) { ret += "In context "; ret += quote(ctx); ret += ": "; }
+	switch (t) {
+		case ValueError::multiple_occurences: ret += "multiple occurences: "; break;
+		case ValueError::invalid_default: x = "default ";
+		case ValueError::invalid_value:       
+			ret += quote(value); 
+			ret += " invalid ";
+			ret += x;
+			ret += "value for: ";
+			break;
+		default: ret += "unknown error in: ";
+	};
+	ret += quote(key);
+	return ret;
+}
+SyntaxError::SyntaxError(Type t, const std::string& key) 
+	: Error(format(t, key)) 
+	, key_(key)
+	, type_(t) {
+}
+ContextError::ContextError(const std::string& ctx, Type t, const std::string& key, const std::string& alt)
+	: Error(format(t, ctx, key, alt))
+	, ctx_(ctx)
+	, key_(key)
+	, type_(t) {
+}
+ValueError::ValueError(const std::string& ctx, Type t, const std::string& opt, const std::string& value)
+	: Error(format(t, ctx, opt, value))
+	, ctx_(ctx)
+	, key_(opt)
+	, value_(value)
+	, type_(t) {
+}
+
 }
